package com.mdotm.android.admobadapter;

import android.app.Activity;
import android.content.Context;
import android.os.Build;
import android.util.Log;
import android.view.View;

import com.google.ads.AdRequest.ErrorCode;
import com.google.ads.AdSize;
import com.google.ads.mediation.MediationAdRequest;
import com.google.ads.mediation.MediationBannerAdapter;
import com.google.ads.mediation.MediationBannerListener;
import com.google.ads.mediation.MediationInterstitialAdapter;
import com.google.ads.mediation.MediationInterstitialListener;
import com.mdotm.android.listener.MdotMAdEventListener;
import com.mdotm.android.model.MdotMAdRequest;
import com.mdotm.android.view.MdotMAdView;
import com.mdotm.android.view.MdotMInterstitial;

/**
 * Skeleton Adapter for AdMob Mediation
 */
public final class MdotMAdapter
		implements
		MediationBannerAdapter<MdotMNetworkExtras, MdotMServerParameters>,
		MediationInterstitialAdapter<MdotMNetworkExtras, MdotMServerParameters>,
		MdotMAdEventListener {

	/*
	 * Callback listeners. This class handles both in-activity (banner) and
	 * interstitial ads, so it listens for both
	 */
	private MediationBannerListener bannerListener;
	private MediationInterstitialListener interstitialListener;
	MdotMAdView bannerView;
	MdotMInterstitial interstitialView;
	Activity activity;

	/*
	 * ------------------------------------------------------------------------
	 * MediationAdapter Implementation
	 * ------------------------------------------------------------------------
	 */

	/*
	 * These methods let the mediation layer know what data types are used for
	 * server-side parameters and publisher "extras"
	 */
	public Class<MdotMNetworkExtras> getAdditionalParametersType() {
		return MdotMNetworkExtras.class;
	}

	public Class<MdotMServerParameters> getServerParametersType() {
		return MdotMServerParameters.class;
	}

	/*
	 * Ad Requests
	 */
	public void requestBannerAd(MediationBannerListener listener,
			Activity activity, MdotMServerParameters serverParameters,
			AdSize adSize, MediationAdRequest mediationAdRequest,
			MdotMNetworkExtras extras) {
		bannerListener = listener;
		try {
			if (Build.VERSION.SDK_INT >= 7) {

				bannerView = new MdotMAdView((Context) activity);
				MdotMAdRequest request = new MdotMAdRequest();
				// Adsize check for available adsize and return nofill if not
				// there

				AdSize bestAdSize = adSize.findBestSize(new AdSize(320, 50),
						new AdSize(300, 50), new AdSize(300, 250), new AdSize(
								320, 480), new AdSize(480, 320), new AdSize(
								468, 60), new AdSize(728, 90));

				if (bestAdSize == null) {
					bannerListener.onFailedToReceiveAd(this,
							ErrorCode.INVALID_REQUEST);
					Log.e("",
							"No Ad Retrieved -- MdotM doesn't support requested ad size ["
									+ adSize + "]");
					return;
				} else {

					String adDimension = bestAdSize.getWidth() + ","
							+ bestAdSize.getHeight();
					request.setAdSize(adDimension);
					// app key from mediation
					request.setAppKey(serverParameters.appKey);

					// Test mode
					if (mediationAdRequest.isTesting()) {
						request.setTestMode("1");
					} else {
						request.setTestMode("0");
					}

					// enable caching
					if (extras != null) {
						request.setEnableCaching(extras.isEnableCaching());

					} else {
						request.setEnableCaching(false);
					}
					bannerView.loadBannerAd(this, request);
				}
			} else {
				if (bannerListener != null) {
					bannerListener.onFailedToReceiveAd(MdotMAdapter.this,
							ErrorCode.NO_FILL);
				}
			}
		} catch (Exception e) {
			Log.e("MdotMAdapter",
					"Android api version is less than 7 (i.e 2.1)");
			if (bannerListener != null) {
				bannerListener.onFailedToReceiveAd(MdotMAdapter.this,
						ErrorCode.NO_FILL);
			}
		}
	}

	public void requestInterstitialAd(MediationInterstitialListener listener,
			Activity activity, MdotMServerParameters serverParameters,
			MediationAdRequest mediationAdRequest, MdotMNetworkExtras extras) {
		interstitialListener = listener;
		try {
			if (Build.VERSION.SDK_INT >= 7) {

				this.activity = activity;
				MdotMAdRequest request = new MdotMAdRequest();
				// Adsize check for available adsize and return nofill if not
				// there
				// app key from mediation
				request.setAppKey(serverParameters.appKey);

				// Test mode
				if (mediationAdRequest.isTesting()) {
					request.setTestMode("1");
				} else {
					request.setTestMode("0");
				}

				// enable caching
				if (extras != null) {
					request.setEnableCaching(extras.isEnableCaching());

				} else {
					request.setEnableCaching(false);
				}

				interstitialView = new MdotMInterstitial(activity);

				interstitialView.loadInterstitial(this, request);
			} else {
				if (interstitialListener != null) {
					interstitialListener.onFailedToReceiveAd(MdotMAdapter.this,
							ErrorCode.NO_FILL);
				}
			}
		} catch (Exception e) {
			Log.e("MdotMAdapter",
					"Android api version is less than 7 (i.e 2.1)");
			if (interstitialListener != null) {
				interstitialListener.onFailedToReceiveAd(MdotMAdapter.this,
						ErrorCode.NO_FILL);
			}
		}
	}

	public void showInterstitial() {
		if (interstitialView != null && this.activity != null) {
			interstitialView.showInterstitial(this.activity);
		} else {
			Log.e("MdotMAdapter",
					"Unable to show interstitial because either interstitialView is null or activity context is null");
		}
	}

	public void destroy() {
		// Log.i("MdotMAdapter", "Destory is called");
		if (bannerView != null) {
			bannerView.endAdSession();
		}
	}

	public View getBannerView() {
		return bannerView;
	}

	public void onFailedToReceiveBannerAd() {
		// Log.e("MdotMAdmobAdapter", "Banner failed to receive");
		if (bannerListener != null) {
			bannerListener.onFailedToReceiveAd(MdotMAdapter.this,
					ErrorCode.NO_FILL);
		}

	}

	public void onBannerAdClick() {
		if (bannerListener != null) {
			// Log.e("", "banner clicked");
			bannerListener.onClick(MdotMAdapter.this);
		} else {
			Log.e("MdotMAdmobAdapter",
					"Listner is null not able report banner click");
		}
	}

	public void onReceiveInterstitialAd() {
		// Log.i("MdotMAdmobAdapter", "receive Interstitial");

		if (interstitialListener != null) {
			interstitialListener.onReceivedAd(MdotMAdapter.this);
		} else {
			Log.e("MdotMAdmobAdapter", "IAd received but listener is null");
		}
	}

	@Override
	public void onReceiveBannerAd() {
		// Log.i("MdotMAdmobAdapter", "Banner received");

		if (bannerListener != null) {
			bannerView.setVisibility(View.VISIBLE);
			bannerView.bringToFront();
			bannerListener.onReceivedAd(MdotMAdapter.this);
		} else {
			Log.e("MdotMAdmobAdapter", "B-Ad received but listener is null");
		}

	}

	@Override
	public void onFailedToReceiveInterstitialAd() {
		// Log.e("MdotMAdmobAdapter", "Failed interstitial");
		if (interstitialListener != null) {
			interstitialListener.onFailedToReceiveAd(MdotMAdapter.this,
					ErrorCode.NO_FILL);
		}

	}

	@Override
	public void onInterstitialAdClick() {
		// Log.e("", " interstitial click");
	}

	@Override
	public void onLeaveApplicationFromBanner() {
		// Log.e("", " leave banner");
		if (bannerListener != null) {
			bannerListener.onLeaveApplication(MdotMAdapter.this);
		}

	}

	@Override
	public void onLeaveApplicationFromInterstitial() {
		if (interstitialListener != null) {
			interstitialListener.onLeaveApplication(MdotMAdapter.this);
		}
	}

	@Override
	public void willShowInterstitial() {
		// TODO Auto-generated method stub

	}

	@Override
	public void didShowInterstitial() {
		if (interstitialListener != null) {
			interstitialListener.onPresentScreen(MdotMAdapter.this);
		}

	}

	@Override
	public void onBannerDismissScreen() {
		if (bannerListener != null) {
			// Log.e("", "OnDismiss");
			bannerListener.onDismissScreen(MdotMAdapter.this);
		}
	}

	@Override
	public void onInterstitialDismiss() {
		if (interstitialListener != null) {
			// Log.e("", "OnDismiss");
			interstitialListener.onDismissScreen(MdotMAdapter.this);
		}
	}

	// @Override
	// public void didExpandAd() {
	// // TODO Auto-generated method stub
	//
	// }
	//
	// @Override
	// public void didResizeAd() {
	// // TODO Auto-generated method stub
	//
	// }
	//
	// @Override
	// public void didChangeToDefaultSize() {
	// // TODO Auto-generated method stub
	//
	// }

}
