package com.mdotm.android.database;

import java.io.File;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Environment;

import com.mdotm.android.http.MdotMNetworkManager;
import com.mdotm.android.utils.MdotMLogger;

public class MdotMCacheHandler extends MdotMDBAdapter {
	public static final String TABLE_NAME_AD_CACHE = "AdCache";
	final String FILE_NAME = "FileName";
	final String LAST_USED = "lastUsed";
	// final String FILE_TYPE = "FileType";

	public static final String TABLE_NAME_AD_RESOURCES = "AdHtmlResources";
	final String HTML_FILE_NAME = "HtmlName";
	final String RESOURCE_NAME = "ResourceName";

	final String TABLE_NAME_CACHE_LOCATION = "CacheLocation";
	final String LOCATION = "Location";

	public static final int CACHE_IS_ON_SD_CARD = 1;
	public static final int CACHE_IS_ON_INTERNAL = 2;

	protected static final String DATABASE_NAME = "AdDatabase.db";
	protected static final int DATABASE_VERSION = 1;

	private final String TABLE_CREATE_AD_CACHE = "CREATE TABLE IF NOT EXISTS "
			+ TABLE_NAME_AD_CACHE + " (  " + FILE_NAME + " TEXT NOT NULL,"
			+ LAST_USED + " INTEGER );";

	private final String TABLE_CREATE_RESOURCE_TABLE = "CREATE TABLE IF NOT EXISTS "
			+ TABLE_NAME_AD_RESOURCES
			+ " (  "
			+ HTML_FILE_NAME
			+ " TEXT NOT NULL," + RESOURCE_NAME + " TEXT );";
	private final String TABLE_CREATE_LOCATION_TABLE = "CREATE TABLE IF NOT EXISTS "
			+ TABLE_NAME_CACHE_LOCATION
			+ " ( "
			+ LOCATION
			+ " INTEGER NOT NULL);";

	private static MdotMCacheHandler instance;

	public synchronized static MdotMCacheHandler getInstance(Context appContext) {

		if (instance == null) {
			instance = new MdotMCacheHandler(appContext, DATABASE_NAME,
					DATABASE_VERSION);
		}
		return instance;
	}

	public MdotMCacheHandler(Context context, String DatabaseName, int version) {
		super(context, DatabaseName, version);
		createTable();
	}

	public synchronized void createTable() {
		SQLiteDatabase db = null;
		MdotMLogger.i(this, "creating table");
		try {
			MdotMLogger.i(this, "Create table is called");
			db = getWritableDatabase();
			db.execSQL(TABLE_CREATE_AD_CACHE);
			db.execSQL(TABLE_CREATE_RESOURCE_TABLE);
			db.execSQL(TABLE_CREATE_LOCATION_TABLE);

			MdotMLogger.i(this, "database path is " + db.getPath());
		} catch (Exception e) {
			// //MdotMLogger.e(this, "database table not created exception ",
			// e);
		} finally {
			db.close();
			MdotMLogger.i(this, "DB CLOSED");
		}
	}

	public void insertCache(String fileName, String tableName) throws Exception {
		SQLiteDatabase db = null;
		ContentValues values = new ContentValues();
		values.put(FILE_NAME, fileName);
		values.put(LAST_USED, System.currentTimeMillis());
		// String query = "INSERT INTO " + tableName + " VALUES ( '" + fileName
		// + "', " + System.currentTimeMillis() + ")";
		MdotMLogger.i(this, "Inserting file name " + fileName + " to "
				+ tableName);
		db = getWritableDatabase();
		db.insert(tableName, null, values);
		// db.rawQuery(query, null);
		MdotMLogger.i(this, "Name inserted successfuly");

		if (db != null) {
			db.close();
		}

	}

	/**
	 * deletes least recently used cached file and related resources
	 * 
	 * @param context
	 */
	public boolean deleteCache(Context context, int location) {
		SQLiteDatabase db = null;

		// selecting least recently used file
		String query = "SELECT " + FILE_NAME + " FROM " + TABLE_NAME_AD_CACHE
				+ " ORDER BY " + LAST_USED + " ASC LIMIT 1";
		Cursor cacheCursor = null;
		String fileName = null;
		try {
			db = getWritableDatabase();
			cacheCursor = db.rawQuery(query, null);
			if (cacheCursor != null && cacheCursor.moveToFirst()) {
				// //MdotMLogger.e(this, "The cache cursor is:"
				// + cacheCursor.getCount());
				fileName = cacheCursor.getString(cacheCursor
						.getColumnIndex(FILE_NAME));
				// closing the cursor
				cacheCursor.close();
				String path;
				if (location == CACHE_IS_ON_INTERNAL) {
					path = context.getCacheDir() + "/"
							+ MdotMNetworkManager.MdotM_CACHE_FOLDER + "/";
				} else {
					path = Environment.getExternalStorageDirectory() + "/"
							+ MdotMNetworkManager.MdotM_CACHE_FOLDER + "/";
				}
				if (fileName != null && !fileName.equalsIgnoreCase("null")
						&& fileName.trim().length() > 0) {
					File file = new File(path + fileName);
					if (file.exists()) {
						file.delete();
					}
					// deleting from the cache index table
					int i = db.delete(TABLE_NAME_AD_CACHE, FILE_NAME + "='"
							+ fileName + "'", null);
					// MdotMLogger.e(this, "the deleted file is:" + i + "::"
					// + file.getAbsolutePath());

					// get related resources
					String[] resources = getRelatedResources(fileName);
					if (resources != null) {
						for (String resource : resources) {
							clearCachedResource(resource, context, location,
									fileName);
						}
					}
				}
			}

		} catch (Exception e) {
			// MdotMLogger.e(this, e.getMessage());
			return false;
		} finally {

			if (db != null && db.isOpen()) {
				db.close();
			}
		}

		return true;
	}

	// /**
	// * deleting the resources from cache folder
	// *
	// * @param resources
	// * @param con
	// */
	// private void deleteFromCacheDir(String[] resources, Context con,
	// int location, String fileName) {
	// String path;
	// if (location == CACHE_IS_ON_INTERNAL) {
	// path = con.getCacheDir() + "/"
	// + MdotMNetworkManager.MdotM_CACHE_FOLDER + "/";
	// } else {
	// path = Environment.getExternalStorageDirectory() + "/"
	// + MdotMNetworkManager.MdotM_CACHE_FOLDER + "/";
	// }
	// if (resources != null) {
	// for (String resource : resources) {
	// boolean isReferenced = checkReference(resource, fileName);
	// if (!isReferenced) {
	// File f = new File(path + resource);
	// boolean deleted = f.delete();
	// //MdotMLogger.e(this, "File deleted:" + deleted + "::"
	// + f.getAbsolutePath());
	// }
	// }
	// }
	// }

	/**
	 * deleting cached resources and there references from cache folder and from
	 * resource table
	 * 
	 * @param resource
	 * @param con
	 * @param location
	 * @param mainHtml
	 */
	public void clearCachedResource(String resource, Context con, int location,
			String mainHtml) throws Exception {
		String path;
		if (location == CACHE_IS_ON_INTERNAL) {
			path = con.getCacheDir() + "/"
					+ MdotMNetworkManager.MdotM_CACHE_FOLDER + "/";
		} else {
			path = Environment.getExternalStorageDirectory() + "/"
					+ MdotMNetworkManager.MdotM_CACHE_FOLDER + "/";
		}
		if (resource != null) {
			boolean isReferenced = checkReference(resource, mainHtml);
			if (!isReferenced) {
				File f = new File(path + resource);
				boolean deleted = f.delete();
				// MdotMLogger.e(this, "File deleted:" + deleted + "::"
				// + f.getAbsolutePath());
			}
			deleteFileFromResourceTable(resource, mainHtml);
			String[] subResources = getRelatedResources(resource);
			if (subResources != null) {
				for (String subSource : subResources) {
					clearCachedResource(subSource, con, location, resource);
				}

			}

		}

	}

	public String[] getRelatedResources(String resource) {
		SQLiteDatabase db = null;
		String[] resources = null;
		Cursor cursor = null;
		try {
			db = getWritableDatabase();
			cursor = db.query(TABLE_NAME_AD_RESOURCES,
					new String[] { RESOURCE_NAME }, HTML_FILE_NAME + "= '"
							+ resource + "'", null, null, null, null);

			if (cursor.moveToFirst()) {
				int i = 0;
				resources = new String[cursor.getCount()];
				do {
					resources[i++] = cursor.getString(0);
				} while (cursor.moveToNext());
			}
		} catch (Exception e) {
			// TODO: handle exception
		} finally {
			if (cursor != null && !cursor.isClosed()) {
				cursor.close();
			}

			if (db != null && db.isOpen()) {
				db.close();
			}
		}
		return resources;
	}

	private boolean checkReference(String resource, String fileName) {
		if (fileName != null) {
			SQLiteDatabase db = null;
			Cursor cursor = null;
			try {
				db = getWritableDatabase();
				cursor = db.query(TABLE_NAME_AD_RESOURCES,
						new String[] { HTML_FILE_NAME }, HTML_FILE_NAME
								+ "!= '" + fileName + "' AND " + RESOURCE_NAME
								+ " = '" + resource + "'", null, null, null,
						null);
				if (cursor != null && cursor.getCount() > 0) {
					return true;
				}
			} catch (Exception e) {
				// TODO: handle exception
			} finally {
				if (cursor != null && !cursor.isClosed()) {
					cursor.close();
				}
				if (db != null && db.isOpen()) {
					db.close();
				}
			}
		}
		return false;

	}

	/**
	 * Checks for cache status in the location index table
	 * 
	 * @return cache location
	 */
	public int getCacheLocation() {
		SQLiteDatabase db = null;
		Cursor cursor = null;
		try {
			db = getWritableDatabase();
			cursor = db.query(TABLE_NAME_CACHE_LOCATION,
					new String[] { LOCATION }, null, null, null, null, null);
			if (cursor != null && cursor.getCount() > 0) {
				MdotMLogger.i(
						this,
						"location column index  "
								+ cursor.getColumnIndex(LOCATION)
								+ " row count " + cursor.getCount());
				cursor.moveToFirst();
				int location = cursor.getInt(cursor.getColumnIndex(LOCATION));
				return location;
			}
		} catch (Exception e) {
			// MdotMLogger.e(this, "Exception while checking cache "
			// + e.getMessage());
		} finally {
			if (cursor != null && !cursor.isClosed()) {
				cursor.close();
			}
			if (db != null && db.isOpen()) {
				db.close();
			}
		}
		return 0;
	}

	public boolean insertCacheLocation(int location) {
		boolean inserted = false;
		SQLiteDatabase db = null;
		ContentValues values = new ContentValues();
		values.put(LOCATION, location);
		MdotMLogger.i(this, "Inserting location name " + location + " to "
				+ TABLE_NAME_CACHE_LOCATION);
		try {
			db = getWritableDatabase();
			db.delete(TABLE_NAME_CACHE_LOCATION, null, null);
			db.insert(TABLE_NAME_CACHE_LOCATION, null, values);
			// db.rawQuery(query, null);
			MdotMLogger.i(this, "location inserted successfuly");
			inserted = true;
		} catch (Exception e) {
			inserted = false;
			// MdotMLogger.e(this, e.getMessage());
			MdotMLogger.i(this, "Exception while inserting");
		} finally {
			if (db != null) {
				db.close();
			}
		}
		return inserted;
	}

	public String[] getAllFileFromCacheTable() {
		SQLiteDatabase db = null;
		Cursor cacheCursor = null;
		int i = 0;
		String[] filesFromCacheTable = null;
		try {
			db = getWritableDatabase();
			cacheCursor = db.query(TABLE_NAME_AD_CACHE,
					new String[] { FILE_NAME }, null, null, null, null, null);
			if (cacheCursor != null && cacheCursor.getCount() > 0) {
				filesFromCacheTable = new String[cacheCursor.getCount()];
				if (cacheCursor.moveToFirst()) {
					do {
						filesFromCacheTable[i++] = cacheCursor.getString(0);
					} while (cacheCursor.moveToNext());
				} else {
					filesFromCacheTable = null;
				}
			}
		} catch (Exception e) {
			filesFromCacheTable = null;
		} finally {
			if (cacheCursor != null && !cacheCursor.isClosed()) {
				cacheCursor.close();
			}
			if (db != null && db.isOpen()) {
				db.close();
			}
		}

		return filesFromCacheTable;
	}

	public String[] getAllFilesfromResourceTable() {

		SQLiteDatabase db = null;
		Cursor cacheCursor = null;
		int i = 0;
		String[] filesFromResourceTable = null;
		try {
			db = getWritableDatabase();
			cacheCursor = db.query(TABLE_NAME_AD_RESOURCES,
					new String[] { RESOURCE_NAME }, null, null, null, null,
					null);
			if (cacheCursor != null && cacheCursor.getCount() > 0) {
				filesFromResourceTable = new String[cacheCursor.getCount()];
				if (cacheCursor.moveToFirst()) {
					do {
						filesFromResourceTable[i++] = cacheCursor.getString(0);
					} while (cacheCursor.moveToNext());
				} else {
					filesFromResourceTable = null;
				}
			}
		} catch (Exception e) {
			filesFromResourceTable = null;
		} finally {
			if (cacheCursor != null && !cacheCursor.isClosed()) {
				cacheCursor.close();
			}
			if (db != null && db.isOpen()) {
				db.close();
			}
		}

		return filesFromResourceTable;
	}

	public void clearAllTable(Context mContext) {
		SQLiteDatabase db = null;
		try {
			db = getWritableDatabase();
			db.delete(TABLE_NAME_AD_CACHE, null, null);
			db.delete(TABLE_NAME_AD_RESOURCES, null, null);
			db.delete(TABLE_NAME_CACHE_LOCATION, null, null);
		} catch (Exception e) {
			// MdotMLogger.e(this, "exception clearing all table "
			// + e.getMessage());
		} finally {
			if (db != null && db.isOpen()) {
				db.close();
			}
		}
	}

	public void deleteFileFromCacheTable(String substring) {
		SQLiteDatabase db = null;

		try {
			db = getWritableDatabase();
			String delete = "DELETE FROM " + TABLE_NAME_AD_CACHE + " WHERE "
					+ FILE_NAME + " = '" + substring + "'";
			db.execSQL(delete);

			String deleteFromResource = "DELETE FROM "
					+ TABLE_NAME_AD_RESOURCES + " WHERE " + HTML_FILE_NAME
					+ " = '" + substring + "'";
			db.execSQL(deleteFromResource);
		} catch (Exception e) {
			// MdotMLogger.e(this, "Exception while deleting file index");
		} finally {
			if (db != null && db.isOpen()) {
				db.close();
			}
		}
	}

	public void deleteFileFromResourceTable(String substring, String mainString)
			throws Exception {
		SQLiteDatabase db = null;

		try {
			db = getWritableDatabase();

			String deleteFromResource = "DELETE FROM "
					+ TABLE_NAME_AD_RESOURCES + " WHERE " + RESOURCE_NAME
					+ " = '" + substring + "' AND " + HTML_FILE_NAME + "='"
					+ mainString + "'";
			db.execSQL(deleteFromResource);
		} finally {
			if (db != null && db.isOpen()) {
				db.close();
			}
		}

	}

	public void deleteAllCacheFromCacheAndResourceTable(Context mContext,
			int cacheLocation) {
		String path;
		if (cacheLocation == CACHE_IS_ON_INTERNAL) {
			path = mContext.getCacheDir().getAbsolutePath() + "/"
					+ MdotMNetworkManager.MdotM_CACHE_FOLDER + "/";
		} else {
			path = Environment.getExternalStorageDirectory().getAbsolutePath()
					+ "/" + MdotMNetworkManager.MdotM_CACHE_FOLDER + "/";
		}
		String[] cacheFiles = getAllFileFromCacheTable();
		for (String file : cacheFiles) {
			File f = new File(path + file);
			f.delete();

		}
		SQLiteDatabase db = null;
		try {
			db = getWritableDatabase();
			db.delete(TABLE_NAME_AD_CACHE, null, null);
		} catch (Exception e) {
			// MdotMLogger.e(this, "Exception while deleting cache table");
		} finally {
			if (db != null && db.isOpen()) {
				db.close();
			}
		}

		String[] resources = getAllFilesfromResourceTable();
		for (String file : resources) {
			File f = new File(path + file);
			f.delete();
		}

		try {
			db = getWritableDatabase();
			db.delete(TABLE_NAME_AD_RESOURCES, null, null);
		} catch (Exception e) {
			// MdotMLogger.e(this, "Exception while deleting cache table");
		} finally {
			if (db != null && db.isOpen()) {
				db.close();
			}
		}

	}

	public void insertToResourceTable(String resourcename, String mainHtmlUrl) {
		SQLiteDatabase db = null;
		ContentValues values = new ContentValues();
		values.put(HTML_FILE_NAME, mainHtmlUrl);
		values.put(RESOURCE_NAME, resourcename);
		// String query = "INSERT INTO " + tableName + " VALUES ( '" + fileName
		// + "', " + System.currentTimeMillis() + ")";
		MdotMLogger.i(this, "Inserting file name " + resourcename
				+ " of main file " + mainHtmlUrl);
		try {
			db = getWritableDatabase();
			db.insert(TABLE_NAME_AD_RESOURCES, null, values);
			// db.rawQuery(query, null);
			MdotMLogger.i(this, "Name inserted successfuly");
		} catch (Exception e) {
			// MdotMLogger.e(this, e.getMessage());
			MdotMLogger.i(this, "Exception while inserting");
		} finally {
			if (db != null) {
				db.close();
			}
		}

	}

	public String[] getResourceParents(String file) {
		SQLiteDatabase db = null;
		String[] resources = null;
		Cursor cursor = null;
		try {
			db = getWritableDatabase();
			cursor = db.query(TABLE_NAME_AD_RESOURCES,
					new String[] { HTML_FILE_NAME }, RESOURCE_NAME + "= '"
							+ file + "'", null, null, null, null);

			if (cursor.moveToFirst()) {
				int i = 0;
				resources = new String[cursor.getCount()];
				do {
					resources[i++] = cursor.getString(0);
				} while (cursor.moveToNext());
			}
		} catch (Exception e) {
			// TODO: handle exception
		} finally {
			if (cursor != null && !cursor.isClosed()) {
				cursor.close();
			}

			if (db != null && db.isOpen()) {
				db.close();
			}
		}
		return resources;
	}
}
