package com.mdotm.android.http;

import java.io.File;
import java.io.InputStream;

import android.content.Context;
import android.os.Environment;

import com.mdotm.android.database.MdotMCacheHandler;
import com.mdotm.android.model.MdotMAdResponse;
import com.mdotm.android.utils.MdotMLogger;

/**
 * @author Bheema
 * 
 */
public class AdContentDownloader {

	/**
	 * Downloads content of the response
	 * 
	 * @param adResponse
	 * @param isCacheEnabled
	 */
	public void downloadResources(MdotMAdResponse adResponse,
			int cacheLocation, Context context, boolean isCacheEnabled) {
		if (adResponse.getImageUrl() != null
				&& !adResponse.getImageUrl().equalsIgnoreCase("null")
				&& adResponse.getImageUrl().trim().length() > 0) {
			try {
				String filename = adResponse.getImageUrl();
				String ext = filename.substring(filename.lastIndexOf(".") + 1);
				MdotMLogger.i(this, "extension is " + ext);

				// identifying the mime type based on the extension
				setMimeType(ext, adResponse);
				// if (cacheLocation != 0
				// || (cacheLocation == 0 && !adResponse
				// .isRequestForBannerAd())) {
				if (adResponse.getResourceType() == MdotMNetworkManager.imageResource
						|| adResponse.getResourceType() == MdotMNetworkManager.videoResource
						|| adResponse.getResourceType() == MdotMNetworkManager.gifImageResource) {

					MdotMLogger.i(this, "image/video/gifimage resource");
					filename = getFileName(filename);
					File localFile;
					if (cacheLocation != 0 && isCacheEnabled) {
						if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL) {
							localFile = new File(context.getCacheDir() + "/"
									+ MdotMNetworkManager.MdotM_CACHE_FOLDER
									+ "/" + filename);
						} else {
							localFile = new File(
									Environment.getExternalStorageDirectory()
											+ "/"
											+ MdotMNetworkManager.MdotM_CACHE_FOLDER
											+ "/" + filename);
						}
					} else {

						MdotMLogger.i(this, "cache is not enabled");
						if (adResponse.isRequestForBannerAd()) {
							// localFile = new File(context.getCacheDir() + "/"
							// + MdotMNetworkManager.MdotM_CACHE_FOLDER
							// + "/" + filename);

							if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_SD_CARD) {
								localFile = new File(
										Environment
												.getExternalStorageDirectory()
												.getAbsolutePath()
												+ "/"
												+ MdotMNetworkManager.MdotM_CACHE_FOLDER
												+ "/"
												+ MdotMNetworkManager.MdotM_Banner_Temp
												+ "/" + filename);
							} else {
								localFile = new File(
										context.getCacheDir()
												+ "/"
												+ MdotMNetworkManager.MdotM_CACHE_FOLDER
												+ "/"
												+ MdotMNetworkManager.MdotM_Banner_Temp
												+ "/" + filename);
							}
						} else {
							if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_SD_CARD) {

								localFile = new File(
										Environment
												.getExternalStorageDirectory()
												.getAbsolutePath()
												+ "/"
												+ MdotMNetworkManager.MdotM_CACHE_FOLDER
												+ "/"
												+ MdotMNetworkManager.MdotM_Interstitial_Temp
												+ "/" + filename);
							} else {

								localFile = new File(
										context.getCacheDir()
												+ "/"
												+ MdotMNetworkManager.MdotM_CACHE_FOLDER
												+ "/"
												+ MdotMNetworkManager.MdotM_Interstitial_Temp
												+ "/" + filename);
							}
						}
					}

					if (localFile.exists()) {
						MdotMLogger.i(this,
								"Cache exist locally using the same file");
						// File exist in the cache
						adResponse.setImageUrl(localFile.getAbsolutePath());
						adResponse.setCachedLocally(true);
					} else {
						if (isCacheEnabled) {
							// delete from cache table if that file reference is
							// there
							MdotMLogger.i(this, "cache does not exist");
							MdotMCacheHandler.getInstance(context)
									.deleteFileFromCacheTable(filename);
						}
						InputStream in = null;
						if (isCacheEnabled) {
							// MdotMLogger.i(this, "cache does not 0 again");
							in = new FileDownloader().downloadFile(context,
									adResponse.getImageUrl());
							// in = downloadFile(adResponse);
							if (in == null) {
								MdotMLogger.i(this, "unable to download file");
								// if (!adResponse.isRequestForBannerAd()) {
								// adResponse = null;
								adResponse.setStatus(0);

								// } else {
								adResponse.setCachedLocally(false);
								// }
								return;
							} else if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL
									&& adResponse.getResourceType() == MdotMNetworkManager.videoResource) {
								// video can not be played from internal
								// cache
								// so video is not caching
								// if (!adResponse.isRequestForBannerAd()) {
								MdotMLogger
										.e(this,
												"Video ad can not be played from internal cache, so not caching the ad");
								// adResponse = null;
								adResponse.setStatus(0);
								// } else {
								adResponse.setCachedLocally(false);
								// }
								return;
							} else {

								localFile.createNewFile();
								boolean isFilewrote = new FileDownloader()
										.writeStreamToFile(in, localFile);
								if (isFilewrote) {

									adResponse.setImageUrl(localFile
											.getAbsolutePath());
									MdotMCacheHandler
											.getInstance(context)
											.insertCache(
													filename,
													MdotMCacheHandler.TABLE_NAME_AD_CACHE);
									adResponse.setCachedLocally(true);
								} else {
									// if (!adResponse.isRequestForBannerAd()) {
									// adResponse = null;
									adResponse.setStatus(0);
									// } else {
									adResponse.setCachedLocally(false);
									// }
									return;
								}

							}
						} else {
							if (!(adResponse.getResourceType() == MdotMNetworkManager.videoResource && cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL)) {
								in = new FileDownloader().downloadFile(context,
										adResponse.getImageUrl());
								// in = downloadFile(adResponse.);

								if (in == null) {
									MdotMLogger
											.i(this,
													"returning response as it is. Since unable to download file");
									// if (!adResponse.isRequestForBannerAd()) {
									// adResponse = null;
									adResponse.setStatus(0);
									// } else {
									adResponse.setCachedLocally(false);
									// }
									return;
								} else {

									localFile.createNewFile();
									boolean isFilewrote = new FileDownloader()
											.writeStreamToFile(in, localFile);

									if (isFilewrote) {
										adResponse.setImageUrl(localFile
												.getAbsolutePath());
										// MdotMCacheHandler
										// .getInstance(context)
										// .insertCache(
										// filename,
										// MdotMCacheHandler.TABLE_NAME_AD_CACHE);
										adResponse.setCachedLocally(true);
									} else {
										// if
										// (!adResponse.isRequestForBannerAd())
										// {
										// adResponse = null;
										adResponse.setStatus(0);
										// } else {
										adResponse.setCachedLocally(false);
										// returning response url as it
										// is
										// }

										return;
									}
								}

							} else {
								// returning response as it is
								MdotMLogger.i(this, "video resource");
								// if (!adResponse.isRequestForBannerAd()) {
								adResponse.setStatus(0);
								MdotMLogger
										.e(this,
												"Its a video ad. can not play from internal cache");
								// adResponse = null;
								// } else {
								adResponse.setCachedLocally(false);
								// returning response url as it is
								// }
								return;
							}
						}
					}

				} else {
					MdotMLogger.i(this, "html response");
					// new HtmlProcessor().ParseHtml(adResponse,
					// cacheLocation,
					// context);
					return;
				}
				// } else {
				// MdotMLogger.i(this, "banner not caching");
				// adResponse.setCachedLocally(false);
				// }
				// return;

			} catch (Exception e) {
				MdotMLogger.e(this,
						"Exception while downloading file " + e.getMessage());
				// e.printStackTrace();
				// if (!adResponse.isRequestForBannerAd()) {
				// adResponse = null;
				adResponse.setStatus(0);
				// } else {
				adResponse.setCachedLocally(false);
				// returning response url as it is
				// }
				return;
			}
		} else {
			MdotMLogger.e(this, "Image url is null");
			// image url is null sending response null
			adResponse.setStatus(0);
			// adResponse = null;
		}
	}

	/**
	 * identifies the file type based on extension. File type may be image,
	 * video, html
	 * 
	 * @param ext
	 * @param adResponse
	 */
	private void setMimeType(String ext, MdotMAdResponse adResponse) {
		if (ext.equalsIgnoreCase("jpg") || ext.equalsIgnoreCase("png")
				|| ext.equalsIgnoreCase("jpeg")) {
			adResponse.setResourceType(MdotMNetworkManager.imageResource);
		} else if (ext.equalsIgnoreCase("mp4") || ext.equalsIgnoreCase("m4v")
				|| ext.equalsIgnoreCase("avi") || ext.equalsIgnoreCase("3gp")) {
			adResponse.setResourceType(MdotMNetworkManager.videoResource);
		} else if (ext.equalsIgnoreCase("gif")) {
			adResponse.setResourceType(MdotMNetworkManager.gifImageResource);
		} else if (!ext.contains("webarchive")) {
			adResponse.setResourceType(MdotMNetworkManager.htmlResource);
		}

	}

	private String getFileName(String fileName) {
		String mainHtmlname = fileName.replace("\\", "");
		mainHtmlname = mainHtmlname.replace("/", "");
		mainHtmlname = mainHtmlname.replace(".", "");
		mainHtmlname = mainHtmlname.replace(":", "");
		mainHtmlname = mainHtmlname.replace("?", "");
		return mainHtmlname;
	}

}
