package com.mdotm.android.http;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Environment;

import com.mdotm.android.database.MdotMCacheHandler;
import com.mdotm.android.model.HtmlResourceModel;
import com.mdotm.android.utils.MdotMLogger;

/**
 * @author Bheema
 * 
 */
public class FileDownloader {
	public InputStream downloadFile(Context context, String url) {
		InputStream in = null;
		try {
			if (isNetworkAvailable(context)) {
				HttpParams httpParameters = new BasicHttpParams();
				// Set the timeout in milliseconds until a connection is
				// established.
				// The default value is zero, that means the timeout is not
				// used.
				int timeoutConnection = 5000;
				HttpConnectionParams.setConnectionTimeout(httpParameters,
						timeoutConnection);
				// Set the default socket timeout (SO_TIMEOUT)
				// in milliseconds which is the timeout for waiting for data.
				int timeoutSocket = 10000;
				HttpConnectionParams
						.setSoTimeout(httpParameters, timeoutSocket);
				// httpClient.getParams().setParameter(
				// HttpConnectionParams.CONNECTION_TIMEOUT,
				// Integer.valueOf(CONNECTION_TIME_OUT));
				// httpClient.getParams().set
				HttpClient httpClient = new DefaultHttpClient(httpParameters);
				// HttpClient httpClient = new DefaultHttpClient();
				// httpClient.getParams().setParameter(
				// HttpConnectionParams.CONNECTION_TIMEOUT,
				// Integer.valueOf(5000));
				HttpResponse response = httpClient.execute(new HttpGet(url));
				if (response.getStatusLine().getStatusCode() == 200) {
					in = response.getEntity().getContent();
				} else {
					MdotMLogger.e(this,
							"Failed to download file. response code "
									+ response.getStatusLine().getStatusCode());
					in = null;
				}

			} else {
				MdotMLogger.e(this, "n/w not available to download file");
				in = null;
			}
		} catch (MalformedURLException e) {
			MdotMLogger.e(this,
					"MalformedURLException exception while downloading file "
							+ e.getMessage());
			// e.printStackTrace();
			in = null;
		} catch (IOException e) {
			MdotMLogger.e(this, "IOException exception while downloading file "
					+ e.getMessage());
			// e.printStackTrace();
			in = null;
		}
		return in;
	}

	public void downloadHtmlResources(Context context,
			HtmlResourceModel htmlResourceModel, int cacheLocation) {
		// for (HtmlResourceModel htmlResourceModel : sourceList) {
		MdotMLogger.i(this,
				"Resolved link is " + htmlResourceModel.getResolvedLink());
		if (htmlResourceModel.getAction() == HtmlProcessor.DOWNLOAD_RESOURCE) {
			try {
				// boolean isVideoResource = false;
				// isVideoResource = isVideoResource(htmlResourceModel
				// .getResolvedLink());
				String fileLocalName = getFileName(htmlResourceModel
						.getResolvedLink());
				htmlResourceModel.setCachedFileName(fileLocalName);

				File localFile;
				if (cacheLocation != 0) {

					if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL) {
						localFile = new File(context.getCacheDir() + "/"
								+ MdotMNetworkManager.MdotM_CACHE_FOLDER + "/"
								+ fileLocalName);
					} else {
						localFile = new File(
								Environment.getExternalStorageDirectory()
										+ "/"
										+ MdotMNetworkManager.MdotM_CACHE_FOLDER
										+ "/" + fileLocalName);
					}
				} else {
					localFile = new File(context.getCacheDir() + "/"
							+ MdotMNetworkManager.MdotM_CACHE_FOLDER + "/"
							+ fileLocalName);
				}

				if (localFile.exists()) {
					MdotMLogger.i(this,
							"Cache exist locally using the same file");
					// File exist in the cache so returning path
					htmlResourceModel.setAbsoluteCachedPath(localFile
							.getAbsolutePath());
					// return localFile.getAbsolutePath();
				} else {
					// delete from resource table if that file reference is
					// there
					MdotMCacheHandler.getInstance(context)
							.deleteFileFromCacheTable(
									htmlResourceModel.getCachedFileName());
					InputStream in = null;
					if (cacheLocation != 0) {
						in = downloadFile(context,
								htmlResourceModel.getResolvedLink());
						if (in == null) {
							MdotMLogger
									.i(this,
											"returning response as it is. Since unable to download file");
							htmlResourceModel.setAbsoluteCachedPath(null);
							// return null;
						} else if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL
								&& htmlResourceModel.isVideo()) {
							// video can not be played from internal cache
							// so video is not caching
							htmlResourceModel.setAbsoluteCachedPath(null);
							// return null;
						} else {

							localFile.createNewFile();
							boolean isFilewrote = writeStreamToFile(in,
									localFile);
							if (isFilewrote) {
								htmlResourceModel
										.setAbsoluteCachedPath(localFile
												.getAbsolutePath());
								// MdotMCacheHandler.getInstance(mContext)
								// .insertToResourceTable(resourcename,
								// mainHtmlUrl);
								// return localFile.getAbsolutePath();
							} else {
								htmlResourceModel.setAbsoluteCachedPath(null);
								// returning response url as it is
								// return null;
							}

						}
					} else {
						if (!htmlResourceModel.isVideo()) {
							in = downloadFile(context,
									htmlResourceModel.getResolvedLink());
							if (in == null) {
								MdotMLogger
										.i(this,
												"returning response as it is. Since unable to download file");
								htmlResourceModel.setAbsoluteCachedPath(null);
								// return null;
							} else {

								localFile.createNewFile();
								boolean isFilewrote = writeStreamToFile(in,
										localFile);
								if (isFilewrote) {
									htmlResourceModel
											.setAbsoluteCachedPath(localFile
													.getAbsolutePath());
									// MdotMCacheHandler.getInstance(mContext)
									// .insertToResourceTable(resourcename,
									// mainHtmlUrl);
									// return localFile.getAbsolutePath();
								} else {
									htmlResourceModel
											.setAbsoluteCachedPath(null);
									// returning response url as it is
									// return null;
								}
							}

						} else {
							htmlResourceModel.setAbsoluteCachedPath(null);
							// returning response as it is video not caching
							// on
							// internal memory
							// return null;
						}
					}
					if (in != null) {
						in.close();
					}
				}

			} catch (Exception e) {
				htmlResourceModel.setAbsoluteCachedPath(null);
				// e.printStackTrace();
				// MdotMLogger.e(this,
				// "Exception while downloading html resource "
				// + e.getMessage());
			}
		} else {

		}
		// }
	}

	private boolean isNetworkAvailable(Context context) {
		ConnectivityManager cm = (ConnectivityManager) context
				.getApplicationContext().getSystemService(
						Context.CONNECTIVITY_SERVICE);
		NetworkInfo ni = cm.getActiveNetworkInfo();
		if (ni != null && ni.isConnected()) {
			return true;
		} else {
			return false;
		}
	}

	private String getFileName(String fileName) {
		fileName = fileName.replace("\\", "");
		fileName = fileName.replace("/", "");
		fileName = fileName.replace(".", "");
		fileName = fileName.replace(":", "");
		fileName = fileName.replace("?", "");
		return fileName;
	}

	public boolean writeStreamToFile(InputStream in, File localFile) {
		FileOutputStream out;
		MdotMLogger.i(this, "writing stream to file");
		try {
			out = new FileOutputStream(localFile);
			int read = 0;
			byte[] bytes = new byte[3072];
			while ((read = in.read(bytes)) != -1) {
				out.write(bytes, 0, read);
				MdotMLogger.i(this, "reading file ");
			}
			in.close();
			out.flush();
			out.close();
			MdotMLogger.i(this, "successfully finished writing");
			return true;
		} catch (FileNotFoundException e) {
			MdotMLogger.e(this, "FileNotfound exception while writting file "
					+ e.getMessage());
			// e.printStackTrace();
			localFile.delete();
		} catch (IOException e) {
			// e.printStackTrace();
			localFile.delete();
			MdotMLogger.e(this, "IOException exception while writting file "
					+ e.getMessage());
		}
		return false;
	}
}
