package com.mdotm.android.http;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.HttpConnectionParams;

import android.content.Context;
import android.os.Environment;

import com.mdotm.android.database.MdotMCacheHandler;
import com.mdotm.android.model.HtmlResourceModel;
import com.mdotm.android.model.MdotMAdResponse;
import com.mdotm.android.utils.MdotMLogger;

/**
 * @author Bheema
 * 
 */
public class HtmlProcessor {
	private String baseUrl = null;
	private String mHtmlPageString = null;

	protected static int DOWNLOAD_RESOURCE = 1;
	protected static int RESOLVE_URL = 2;

	// resource types
	protected static int RESOURCE_CSS = 1;
	protected static int RESOURCE_JS = 2;
	protected static int RESOURCE_VIDEO = 3;
	protected static int RESOURCE_AUDIO = 4;
	protected static int RESOURCE_HTML = 5;
	protected static int RESOURCE_OTHER = 6;

	public void ParseHtml(MdotMAdResponse adResponse, int cacheLocation,
			Context context) {
		MdotMLogger.d(this, "parsing html response");
		String htmlFileLocalName = getFileName(adResponse.getImageUrl());
		File localFile;
		if (cacheLocation != 0) {
			if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL) {
				localFile = new File(context.getCacheDir() + "/"
						+ MdotMNetworkManager.MdotM_CACHE_FOLDER + "/"
						+ htmlFileLocalName);
			} else {
				localFile = new File(Environment.getExternalStorageDirectory()
						+ "/" + MdotMNetworkManager.MdotM_CACHE_FOLDER + "/"
						+ htmlFileLocalName);
			}
		} else {
			localFile = new File(context.getCacheDir() + "/"
					+ MdotMNetworkManager.MdotM_CACHE_FOLDER + "/"
					+ htmlFileLocalName);
		}
		MdotMLogger.d(this, "The local file is:" + localFile);
		if (localFile.exists()) {
			MdotMLogger.i(this, "Cache exist locally using the same file");
			// File exist in the cache
			adResponse.setImageUrl(localFile.getAbsolutePath());
			adResponse.setCachedLocally(true);
		} else {
			// delete from cache table if that file reference is
			// there
			MdotMCacheHandler.getInstance(context).deleteFileFromCacheTable(
					htmlFileLocalName);

			InputStream in;
			try {
				HttpClient httpClient = new DefaultHttpClient();
				httpClient.getParams().setParameter(
						HttpConnectionParams.CONNECTION_TIMEOUT,
						new Integer(5000));
				HttpResponse htmlResponse = httpClient.execute(new HttpGet(
						adResponse.getImageUrl()));
				in = htmlResponse.getEntity().getContent();
				if (in == null) {
					return;
				} else {
					StringBuffer buffer = new StringBuffer();
					byte[] data = new byte[1024];
					int len = 0;
					while (-1 != (len = in.read(data))) {
						buffer.append(new String(data, 0, len));
					}
					mHtmlPageString = buffer.toString();
					MdotMLogger.i(this, "html is " + mHtmlPageString);
					// remove manifest
					removeManifest();

					baseUrl = getBaseUrl();
					if (baseUrl == null) {
						baseUrl = adResponse.getImageUrl();
					}
					// parsing html for tags
					mHtmlPageString = ParseHtmlForTags(htmlFileLocalName,
							mHtmlPageString, cacheLocation, context);

					// writting main html to cache folder

					boolean writeSuccess = writeStringToLocalFile(
							mHtmlPageString, localFile);
					if (!writeSuccess) {
						revertHtmlProcessing(htmlFileLocalName, context,
								cacheLocation);
						adResponse.setCachedLocally(false);
					} else {

						try {
							MdotMCacheHandler.getInstance(context).insertCache(
									htmlFileLocalName,
									MdotMCacheHandler.TABLE_NAME_AD_CACHE);
							adResponse.setImageUrl(localFile.getAbsolutePath());
							adResponse.setCachedLocally(true);
						} catch (Exception e) {
							revertHtmlProcessing(htmlFileLocalName, context,
									cacheLocation);
							adResponse.setCachedLocally(false);

							// e.printStackTrace();
						}
					}

				}
			} catch (MalformedURLException e) {
				revertHtmlProcessing(htmlFileLocalName, context, cacheLocation);
				adResponse.setCachedLocally(false);
				return;
			} catch (IOException e) {
				revertHtmlProcessing(htmlFileLocalName, context, cacheLocation);
				adResponse.setCachedLocally(false);
				// e.printStackTrace();
			}
		}
	}

	private void revertHtmlProcessing(String htmlFileLocalName,
			Context context, int cacheLocation) {
		// delete all the resource from resource table and from cachedir
		String[] resources = MdotMCacheHandler.getInstance(context)
				.getRelatedResources(htmlFileLocalName);
		if (resources != null) {
			for (String resource : resources) {
				try {
					MdotMCacheHandler.getInstance(context)
							.clearCachedResource(resource, context,
									cacheLocation, htmlFileLocalName);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					// e.printStackTrace();
				}
			}
		}

	}

	private boolean writeStringToLocalFile(String fileString, File localFile) {
		Writer writer;
		boolean writeSuccess = false;
		try {
			writer = new BufferedWriter(new FileWriter(localFile));
			writer.write(fileString);
			writer.close();
			writeSuccess = true;
		} catch (IOException e) {
			// e.printStackTrace();
		}

		return writeSuccess;
	}

	private String ParseHtmlForTags(String fileLocalName, String fileString,
			int cacheLocation, Context context) {
		ArrayList<HtmlResourceModel> sourceList = new ArrayList<HtmlResourceModel>();
		// Identified base url next identify the possible resource
		// attributes
		sourceList = new HtmlTagFinder().getHtmlTags(fileString);
		// resolving urls
		resolveUrl(sourceList);

		// download resources
		for (HtmlResourceModel htmlResourceModel : sourceList) {
			MdotMLogger.i(this, "original url ** "
					+ htmlResourceModel.getOriginallink() + " resolved "
					+ htmlResourceModel.getResolvedLink());
			htmlResourceModel.setVideo(isVideoResource(htmlResourceModel
					.getResolvedLink()));
			if (htmlResourceModel.getAction() == DOWNLOAD_RESOURCE) {
				new FileDownloader().downloadHtmlResources(context,
						htmlResourceModel, cacheLocation);
				if (htmlResourceModel.getresourceType() == RESOURCE_CSS
						&& htmlResourceModel.getAbsoluteCachedPath() != null) {
					try {
						String cssFileString = readFile(htmlResourceModel
								.getAbsoluteCachedPath());
						cssFileString = ParseHtmlForTags(htmlResourceModel
								.getCachedFileName(), cssFileString,
								cacheLocation, context);
						File cssFile = new File(htmlResourceModel
								.getAbsoluteCachedPath());
						// over writtig the existing file with new updated
						// values
						boolean cssWritten = writeStringToLocalFile(
								cssFileString, cssFile);
						if (!cssWritten) {
							ArrayList<HtmlResourceModel> tempList = new ArrayList<HtmlResourceModel>();
							tempList.add(htmlResourceModel);
							clearCachedResources(tempList, context,
									cacheLocation, fileLocalName);
							htmlResourceModel.setAbsoluteCachedPath(null);
						}
					} catch (IOException e) {
						ArrayList<HtmlResourceModel> tempList = new ArrayList<HtmlResourceModel>();
						tempList.add(htmlResourceModel);
						clearCachedResources(tempList, context, cacheLocation,
								fileLocalName);
						htmlResourceModel.setAbsoluteCachedPath(null);
						// e.printStackTrace();
					}

				}
			}
		}

		// update the html page
		String updatedFile = updateReferences(sourceList, fileLocalName,
				fileString, context);
		return updatedFile;

	}

	private void clearCachedResources(ArrayList<HtmlResourceModel> sourceList,
			Context con, int location, String mainHtml) {
		for (HtmlResourceModel htmlResourceModel : sourceList) {

			if (htmlResourceModel.getAction() == DOWNLOAD_RESOURCE
					&& htmlResourceModel.getAbsoluteCachedPath() != null) {
				try {
					MdotMCacheHandler.getInstance(con).clearCachedResource(
							htmlResourceModel.getCachedFileName(), con,
							location, mainHtml);
				} catch (Exception e) {
					// e.printStackTrace();
				}
			}
		}
	}

	private static String readFile(String path) throws IOException {
		FileInputStream stream = new FileInputStream(new File(path));
		try {
			FileChannel fc = stream.getChannel();
			MappedByteBuffer bb = fc.map(FileChannel.MapMode.READ_ONLY, 0, fc
					.size());
			/* Instead of using default, pass in a decoder. */
			return Charset.defaultCharset().decode(bb).toString();
		} finally {
			stream.close();
		}
	}

	/**
	 * update the html/css file with the resolved and cached urls and update the
	 * index table
	 * 
	 * @param sourceList
	 * @param fileString
	 * @param htmlPageString
	 */
	private String updateReferences(ArrayList<HtmlResourceModel> sourceList,
			String mainHtmlName, String fileString, Context context) {
		for (HtmlResourceModel htmlResourceModel : sourceList) {

			if (htmlResourceModel.getAction() == DOWNLOAD_RESOURCE
					&& htmlResourceModel.getAbsoluteCachedPath() != null) {
				MdotMCacheHandler.getInstance(context).insertToResourceTable(
						htmlResourceModel.getCachedFileName(), mainHtmlName);
				fileString = updateMainHtml(
						htmlResourceModel.getOriginallink(), htmlResourceModel
								.getAbsoluteCachedPath(), fileString);
			} else {
				fileString = updateMainHtml(
						htmlResourceModel.getOriginallink(), htmlResourceModel
								.getResolvedLink(), fileString);
			}
		}
		return fileString;
	}

	private String updateMainHtml(String originalLink, String newLink,
			String fileString) {
		fileString = fileString.replaceAll(originalLink, newLink);
		return fileString;

	}

	private void resolveUrl(ArrayList<HtmlResourceModel> sourceList) {
		for (HtmlResourceModel htmlResourceModel : sourceList) {
			if (htmlResourceModel.getOriginallink() != null) {
				try {
					URI baseUrl = new URI(this.baseUrl);
					URI subUrl = new URI(htmlResourceModel.getOriginallink());
					URI resolvedUri = baseUrl.resolve(subUrl);
					MdotMLogger.i(this, "Base url " + this.baseUrl
							+ " original "
							+ htmlResourceModel.getOriginallink()
							+ " resolved url is " + resolvedUri.toString());
					htmlResourceModel.setResolvedLink(resolvedUri.toString());
				} catch (URISyntaxException e) {
					// TODO Auto-generated catch block
					// e.printStackTrace();
				}
			}

		}
	}

	private void removeManifest() {
		MdotMLogger.i(this, "remove manifest");
		Pattern manifestPattern = Pattern
				.compile("[m|M][a|A][n|N][i|I][f|F][e|E][s|S][t|T]\\s*=\\s*\\\"([^\\\"]+)\\\"");
		Matcher manifestMatcher = manifestPattern.matcher(mHtmlPageString);

		if (manifestMatcher.find()) {
			String manifest = manifestMatcher.group();
			mHtmlPageString = mHtmlPageString.replaceAll(manifest, "");
			MdotMLogger.i(this, "find manifest and removed");
		}

	}

	private String getBaseUrl() {
		MdotMLogger.i(this, "getting base url");
		Pattern baseTagPattern = Pattern.compile("<\\s*base\\s.*?>");
		Matcher srcMatcher = baseTagPattern.matcher(mHtmlPageString);
		if (srcMatcher.find()) {
			MdotMLogger.i(this, "base tag is there");
			String baseString = srcMatcher.group();
			Pattern baseHrefPattern = Pattern
					.compile("[h|H][r|R][e|E][f|F]=\\\"([^\\\"]+)\\\"");
			Matcher hrefMatcher = baseHrefPattern.matcher(baseString);
			if (hrefMatcher.find()) {
				MdotMLogger.i(this, "found bas url ");
				baseUrl = hrefMatcher.group(1);
			} else {
				baseUrl = null;
			}
			// Removing the base tag. Since we are caching html locally
			mHtmlPageString = mHtmlPageString.replaceAll(baseString, "");

		} else {
			baseUrl = null;
		}
		return baseUrl;
	}

	private String getFileName(String fileName) {
		String name = fileName.replace("\\", "");
		name = name.replace("/", "");
		name = name.replace(".", "");
		name = name.replace(":", "");
		name = name.replace("?", "");
		return name;
	}

	private boolean isVideoResource(String resolvedLink) {
		if (resolvedLink != null) {
			MdotMLogger.i(this, "Resoved link is " + resolvedLink);
			String ext = resolvedLink
					.substring(resolvedLink.lastIndexOf(".") + 1);
			MdotMLogger.i(this, "extension is ** " + ext);
			if (ext.equalsIgnoreCase("mp4") || ext.equalsIgnoreCase("m4v")
					|| ext.equalsIgnoreCase("3gp")
					|| ext.equalsIgnoreCase("webm")
					|| ext.equalsIgnoreCase("ogv")
					|| ext.equalsIgnoreCase("ogg")) {
				return true;
			}
		}
		return false;
	}

}
