package com.mdotm.android.http;

import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.mdotm.android.model.HtmlResourceModel;
import com.mdotm.android.utils.MdotMLogger;

/**
 * @author Bheema
 * 
 */
public class HtmlTagFinder {

	String anchorTagRegEx = "<\\s*a\\s.*?>";
	String areaTagRegEx = "<\\s*[a|A][r|R][e|E][a|A]\\s.*?>";
	// String audioTagRegEx =
	// "<\\s*[a|A][u|U][d|D][i|I][o|O]\\s.*?>(.*?)<\\s*/\\s*[a|A][u|U][d|D][i|I][o|O]\\s*>";
	String embedTagRegEx = "<\\s*[e|E][m|M][b|B][e|E][d|D]\\s.*?>";
	String imgTagRegEx = "<\\s*[i|I][m|M][g|G]\\s*.*?>";
	String linkTagRegEx = "<\\s*[l|L][i|I][n|N][k|K]\\s*.*?>";
	String iFrameTagRegEx = "<\\s*[i|I][f|F][r|R][a|A][m|M][e|E]\\s*.*?>";
	String objectTagRegEx = "<\\s*[o|O][b|B][j|J][e|E][c|C][t|T]\\s*.*?>";
	String videoOpenTagRegEx = "<\\s*[v|V][i|I][d|D][e|E][o|O]\\s*.*?>";
	// "<\\s*[v|V][i|I][d|D][e|E][o|O]\\s*.*?>.*?<\\s*/\\s*[v|V][i|I][d|D][e|E][o|O]\\s*>";
	String formTagRegEx = "<\\s*[f|F][o|O][r|R][m|M]\\s.*?>";
	String audioVideoSourceRegEx = "<\\s*[s|S][o|O][u|U][r|R][c|C][e|E]\\s.*?>";
	String scriptTagRegEx = "<\\s*[s|S][c|C][r|R][i|I][p|P][t|T]\\s.*?>";
	String cssImageUrlTagEx = "url\\s*\\('(.*?)'\\)";

	String videoPosterPattern = "[p|P][o|O][s|S][t|T][e|E][r|R]\\s*=\\s*\\\"([^\\\"]+)\\\"";
	String sourcePattern = "[s|S][r|R][c|C]\\s*=\\s*\\\"([^\\\"]+)\\\"";
	String hrefPattern = "[h|H][r|R][e|E][f|F]\\s*=\\s*\\\"([^\\\"]+)\\\"";
	String formActionPattern = "[a|A][c|C][t|T][i|I][o|O][n|N]\\s*=\\s*\\\"([^\\\"]+)\\\"";
	String objectDataPattern = "[d|D][a|A][t|T][a|A]\\s*=\\s*\\\"([^\\\"]+)\\\"";
	String cssImageUrlPattern = "url\\s*\\('(.*?)'\\)";
	String typePattern = "[t|T][y|Y][p|P][e|E]\\s*=\\s*\\\"([^\\\"]+)\\\"";

	ArrayList<HtmlResourceModel> sourceList = new ArrayList<HtmlResourceModel>();

	protected ArrayList<HtmlResourceModel> getHtmlTags(String htmlPageString) {
		// find anchor tag source link
		MdotMLogger.i(this, "Finding for anchor tag");
		findHtmlSources(anchorTagRegEx, hrefPattern, htmlPageString,
				HtmlProcessor.RESOLVE_URL);
		// find area tag source link
		MdotMLogger.i(this, "Finding for area tag");
		findHtmlSources(areaTagRegEx, hrefPattern, htmlPageString,
				HtmlProcessor.RESOLVE_URL);
		// find audio/video source tag source link
		MdotMLogger.i(this, "Finding for audio/video source  tag");
		findHtmlSources(audioVideoSourceRegEx, sourcePattern, htmlPageString,
				HtmlProcessor.DOWNLOAD_RESOURCE);
		// find video poster source link
		MdotMLogger.i(this, "Finding for video open tag");
		findHtmlSources(videoOpenTagRegEx, videoPosterPattern, htmlPageString,
				HtmlProcessor.DOWNLOAD_RESOURCE);
		// find embed tag source link
		MdotMLogger.i(this, "Finding for embed tag");
		findHtmlSources(embedTagRegEx, sourcePattern, htmlPageString,
				HtmlProcessor.RESOLVE_URL);
		// find IMG tag source link
		MdotMLogger.i(this, "Finding for img tag");
		findHtmlSources(imgTagRegEx, sourcePattern, htmlPageString,
				HtmlProcessor.DOWNLOAD_RESOURCE);
		// find link tag source link
		MdotMLogger.i(this, "Finding for link tag");
		findHtmlSources(linkTagRegEx, hrefPattern, htmlPageString,
				HtmlProcessor.DOWNLOAD_RESOURCE);
		// find iframe tag source links
		MdotMLogger.i(this, "Finding for iframe tag");
		findHtmlSources(iFrameTagRegEx, sourcePattern, htmlPageString,
				HtmlProcessor.RESOLVE_URL);
		// find object tag source links
		MdotMLogger.i(this, "Finding for object tag");
		findHtmlSources(objectTagRegEx, objectDataPattern, htmlPageString,
				HtmlProcessor.DOWNLOAD_RESOURCE);
		// find form tag action link
		MdotMLogger.i(this, "Finding for form tag");
		findHtmlSources(formTagRegEx, formActionPattern, htmlPageString,
				HtmlProcessor.RESOLVE_URL);
		// find script tag src link
		findHtmlSources(scriptTagRegEx, sourcePattern, htmlPageString,
				HtmlProcessor.DOWNLOAD_RESOURCE);
		// find css image url link
		findHtmlSources(cssImageUrlTagEx, cssImageUrlPattern, htmlPageString,
				HtmlProcessor.DOWNLOAD_RESOURCE);
		return sourceList;
	}

	private void findHtmlSources(String tagRegEx, String sourceRegEx,
			String htmlPageString, int action) {
		Pattern tagPattern = Pattern.compile(tagRegEx);
		Matcher tagMatcher = tagPattern.matcher(htmlPageString);
		while (tagMatcher.find()) {
			MdotMLogger.i(this, "Tag found ");

			String tagString = tagMatcher.group();
			MdotMLogger.i(this, "Tag String " + tagString);
			String sourcelink = getSourceValue(tagString, sourceRegEx);
			int resourceType = getResourceType(tagString);
			if (sourcelink != null) {
				HtmlResourceModel htmlResourceModel = new HtmlResourceModel();
				htmlResourceModel.setAction(action);
				htmlResourceModel.setOriginallink(sourcelink);
				htmlResourceModel.setresourceType(resourceType);
				sourceList.add(htmlResourceModel);
			}
		}

	}

	private int getResourceType(String tagString) {
		String type = null;
		Pattern sourcePattern = Pattern.compile(typePattern);
		Matcher sourceMatcher = sourcePattern.matcher(tagString);
		if (sourceMatcher.find()) {
			MdotMLogger.i(this, "link found " + sourceMatcher.group()
					+ " 0th position " + sourceMatcher.group(0)
					+ " 1st position " + sourceMatcher.group(1));
			type = sourceMatcher.group(1);
			if (type.contains("css")) {
				return HtmlProcessor.RESOURCE_CSS;
			} else if (type.contains("javascript")) {
				return HtmlProcessor.RESOURCE_JS;
			} else if (type.contains("video")) {
				return HtmlProcessor.RESOURCE_VIDEO;
			} else if (type.contains("audio")) {
				return HtmlProcessor.RESOURCE_AUDIO;
			}
		}

		return HtmlProcessor.RESOURCE_OTHER;
	}

	private String getSourceValue(String tagString, String sourceRegEx) {
		MdotMLogger.i(this, "getSourceValue - tagString" + tagString
				+ " source " + sourceRegEx);
		Pattern sourcePattern = Pattern.compile(sourceRegEx);
		Matcher sourceMatcher = sourcePattern.matcher(tagString);
		if (sourceMatcher.find()) {
			MdotMLogger.i(this, "link found " + sourceMatcher.group()
					+ " 0th position " + sourceMatcher.group(0)
					+ " 1st position " + sourceMatcher.group(1));
			return sourceMatcher.group(1);
		}
		return null;
	}

}
