package com.mdotm.android.http;

import java.io.File;
import java.io.IOException;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;

import android.content.Context;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.os.StatFs;

import com.mdotm.android.database.MdotMCacheHandler;
import com.mdotm.android.listener.MdotMNetworkListener;
import com.mdotm.android.model.MdotMAdRequest;
import com.mdotm.android.model.MdotMAdResponse;
import com.mdotm.android.utils.MdotMLogger;
import com.mdotm.android.utils.MdotMRequestBuilder;

/**
 * @author Bheema
 * 
 */

public class MdotMNetworkManager {
	MdotMNetworkListener networkListener;
	public static String MdotM_CACHE_FOLDER = "MdotMTempCache";
	public static String MdotM_Banner_Temp = "BannerTemp";
	public static String MdotM_Interstitial_Temp = "InterstitialTemp";
	private static boolean enableCaching = false;
	private static MdotMNetworkManager mManager = null;
	/**
	 * minimum there should be 20 MB memory to enable cache
	 */
	private long minMemorySize = 20971520;
	/**
	 * on sd card max cache size 7 MB
	 */
	private int adCacheSizeSDCard = 8340032;

	/**
	 * on internal max cache size 5 MB
	 */
	private int adCacheSizeInternal = 5242880;

	/**
	 * location where cache stored internal or sd card
	 */
	private static int cacheLocation;

	/**
	 * based on available memory we are deciding to cache or not
	 */

	/**
	 * flag to hold cache exist or not
	 */

	/**
	 * Flag to hold sd card status
	 */
	private static boolean isSDCardExist = false;

	// Resource types which we are caching
	public static int imageResource = 1;
	public static int videoResource = 2;
	public static int htmlResource = 3;
	public static int gifImageResource = 4;

	private final int CORE_POOL_SIZE = 1;
	private final int MAX_POOL_SIZE = 1;
	private final int KEEP_ALIVE_TIME_IN_SEC = 30;
	private final int CONNECTION_TIME_OUT = 10000;
	private BlockingQueue<Runnable> worksQueue = new ArrayBlockingQueue<Runnable>(
			1);
	private ThreadPoolExecutor executor;

	private final int HTTP_RESPONSE_OK = 200;

	private Context mContext;

	// private Long mRequestId;
	private Handler mHandler;

	public static MdotMNetworkManager getInstance(Context context,
			Handler handler) {
		if (mManager == null) {
			MdotMLogger.i("", "**Creating new manager");
			mManager = new MdotMNetworkManager(context, handler);
		} else {
			MdotMLogger.i("", "**Using existing manager");
		}
		return mManager;
	}

	private MdotMNetworkManager(Context context, Handler handler) {
		mHandler = handler;
		this.mContext = context;
		setThreadPool();

	}

	/**
	 * Checks the cache status by comparing the values from database to the
	 * existing files.If cache exist then gives the cache location. if not
	 * initializes the cache by checking the available memory and updates DB
	 * index table for tracking.
	 * 
	 * @param requestParams
	 */
	private void checkCacheStatus(MdotMAdRequest requestParams) {
		// if user is enabled cache then initialize cache for permanent caching
		isSDCardExist = checkSDCardState();
		if (enableCaching) {
			// MdotMLogger.i(this, "Cache enabled is " + enableCaching);

			// MdotMLogger.i(this, "sd card exist is " + isSDCardExist);
			cacheLocation = MdotMCacheHandler.getInstance(mContext)
					.getCacheLocation();
			MdotMLogger.i(this, "cache location is " + cacheLocation);
			String path = null;
			if (cacheLocation != 0) {
				manageCacheSize();

				// cache exist checking cache on specified location
				if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL) {
					path = mContext.getCacheDir().getAbsolutePath() + "/"
							+ MdotM_CACHE_FOLDER;
					verifyCache(path);
				} else {
					MdotMLogger.i(this, "cache location is sd card memory");
					if (isSDCardExist) {
						MdotMLogger
								.i(this, "sd card exist is " + isSDCardExist);
						path = Environment.getExternalStorageDirectory()
								.getAbsolutePath() + "/" + MdotM_CACHE_FOLDER;
						verifyCache(path);
					} else {
						MdotMLogger.i(this, "sd card exist is " + isSDCardExist
								+ " clearing tables");
						MdotMCacheHandler.getInstance(mContext).clearAllTable(
								mContext);
						cacheLocation = 0;
					}
				}
			}

			if (cacheLocation != 0) {
				if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_SD_CARD) {
					boolean enoughMemoryAvailable = checkAvailableMemory(cacheLocation);
					MdotMLogger.i(this,
							"location is sd card calculating available memory is available "
									+ enoughMemoryAvailable);
					if (!enoughMemoryAvailable) {

						File cacheFolder = new File(path);
						if (!cacheFolder.exists()) {
							MdotMLogger.i(this,
									"Cache folder doesn't exist on sd card");
							MdotMCacheHandler.getInstance(mContext)
									.clearAllTable(mContext);
							if (!checkAvailableMemory(cacheLocation)) {
								cacheLocation = MdotMCacheHandler.CACHE_IS_ON_INTERNAL;
								if (!checkAvailableMemory(cacheLocation)) {
									cacheLocation = 0;
								}
							}
						} else {
							File[] files = cacheFolder.listFiles();
							clearCahe(files);
							MdotMCacheHandler.getInstance(mContext)
									.clearAllTable(mContext);
							if (!checkAvailableMemory(cacheLocation)) {
								cacheLocation = MdotMCacheHandler.CACHE_IS_ON_INTERNAL;
								if (!checkAvailableMemory(cacheLocation)) {
									cacheLocation = 0;
								}
							}
						}

					}

				} else if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL) {
					boolean enoughMemoryAvailable = checkAvailableMemory(cacheLocation);
					MdotMLogger.i(this,
							"location is internal calculating available memory is available "
									+ enoughMemoryAvailable);

					if (!enoughMemoryAvailable) {

						File cacheFolder = new File(path);
						if (!cacheFolder.exists()) {
							MdotMLogger.i(this,
									"Cache folder doesn't exist on sd card");
							MdotMCacheHandler.getInstance(mContext)
									.clearAllTable(mContext);
							if (!checkAvailableMemory(cacheLocation)) {
								cacheLocation = 0;
							}
						} else {
							File[] files = cacheFolder.listFiles();
							clearCahe(files);
							MdotMCacheHandler.getInstance(mContext)
									.clearAllTable(mContext);
							if (!checkAvailableMemory(cacheLocation)) {
								cacheLocation = 0;
							}
						}

					}
				}
			} else {
				MdotMLogger
						.i(this, "Cache is not there initializing the cache");
				// cache is not there. initializing the cache
				if (isSDCardExist) {
					MdotMLogger.i(this, "sd card exist is " + isSDCardExist);
					boolean enoughMemoryAvailable = checkAvailableMemory(MdotMCacheHandler.CACHE_IS_ON_SD_CARD);
					MdotMLogger.i(this,
							"location is sd card calculating available memory is available "
									+ enoughMemoryAvailable);
					if (enoughMemoryAvailable) {
						cacheLocation = MdotMCacheHandler.CACHE_IS_ON_SD_CARD;
					} else {
						// checking on internal memory
						enoughMemoryAvailable = checkAvailableMemory(MdotMCacheHandler.CACHE_IS_ON_INTERNAL);
						MdotMLogger.i(this,
								"memory is not available on sd card checking on internal memory. is available "
										+ enoughMemoryAvailable);
						if (enoughMemoryAvailable) {
							cacheLocation = MdotMCacheHandler.CACHE_IS_ON_INTERNAL;
						} else {
							cacheLocation = 0;
						}
					}
				} else {
					// checking on internal memory
					boolean enoughMemoryAvailable = checkAvailableMemory(MdotMCacheHandler.CACHE_IS_ON_INTERNAL);
					MdotMLogger.i(this,
							" sd card not there checking on internal memory. is available "
									+ enoughMemoryAvailable);
					if (enoughMemoryAvailable) {
						cacheLocation = MdotMCacheHandler.CACHE_IS_ON_INTERNAL;
					} else {
						cacheLocation = 0;
					}
				}
			}
			File file = null;
			if (cacheLocation != 0) {

				if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL) {
					file = new File(mContext.getCacheDir().getAbsolutePath()
							+ "/" + MdotM_CACHE_FOLDER);
				} else {
					file = new File(Environment.getExternalStorageDirectory()
							.getAbsolutePath() + "/" + MdotM_CACHE_FOLDER);
				}

				if (!file.mkdirs()) {
					if (!file.exists()) {
						if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_SD_CARD) {
							cacheLocation = MdotMCacheHandler.CACHE_IS_ON_INTERNAL;
							file = new File(mContext.getCacheDir()
									.getAbsolutePath()
									+ "/"
									+ MdotM_CACHE_FOLDER);
							MdotMLogger
									.i(this,
											"unable to create folder on sd card so creating on internal memory");
							file.mkdirs();
						}
					} else {
						MdotMLogger.i(this, "File created ");
					}
				}
				if (file.exists()) {
					if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL) {
						MdotMCacheHandler.getInstance(mContext)
								.insertCacheLocation(
										MdotMCacheHandler.CACHE_IS_ON_INTERNAL);
					} else if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_SD_CARD) {
						MdotMCacheHandler.getInstance(mContext)
								.insertCacheLocation(
										MdotMCacheHandler.CACHE_IS_ON_SD_CARD);
					}
				}
				MdotMLogger.i(this,
						"enabled cache location is " + cacheLocation
								+ " file path " + file.getAbsolutePath());
			} else {
				// if enough memory is unavailable then also creating cache
				// folder to cache rich media content temporarily.
				createTemporaryCache(requestParams);
				requestParams.setEnableCaching(false);

				// file = new File(mContext.getCacheDir().getAbsolutePath() +
				// "/"
				// + MdotM_CACHE_FOLDER);
			}

		} else {
			createTemporaryCache(requestParams);

			// cacheLocation = 0;
		}

		// if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL) {
		// MdotMLogger.i(this, "caching on internal memory");
		// } else if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_SD_CARD) {
		// MdotMLogger.i(this, "caching on sd memory");
		//
		// } else {
		// MdotMLogger.i(this, "temporary caching on internal memory");
		//
		// }
	}

	private void createTemporaryCache(MdotMAdRequest requestParams) {
		File file = null;
		if (requestParams.isLoadInterstitial()) {

			if (isSDCardExist) {
				file = new File(Environment.getExternalStorageDirectory()
						.getAbsolutePath()
						+ "/"
						+ MdotM_CACHE_FOLDER
						+ "/"
						+ MdotM_Interstitial_Temp);
				cacheLocation = MdotMCacheHandler.CACHE_IS_ON_SD_CARD;
			} else {
				file = new File(mContext.getCacheDir() + "/"
						+ MdotM_CACHE_FOLDER + "/" + MdotM_Interstitial_Temp);
				cacheLocation = MdotMCacheHandler.CACHE_IS_ON_INTERNAL;
			}

		} else {
			if (isSDCardExist) {
				file = new File(Environment.getExternalStorageDirectory()
						.getAbsolutePath()
						+ "/"
						+ MdotM_CACHE_FOLDER
						+ "/"
						+ MdotM_Banner_Temp);
				cacheLocation = MdotMCacheHandler.CACHE_IS_ON_SD_CARD;
			} else {
				file = new File(mContext.getCacheDir() + "/"
						+ MdotM_CACHE_FOLDER + "/" + MdotM_Banner_Temp);
				cacheLocation = MdotMCacheHandler.CACHE_IS_ON_INTERNAL;
			}

		}

		if (!file.mkdirs()) {
			if (!file.exists()) {
				if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_SD_CARD) {
					cacheLocation = MdotMCacheHandler.CACHE_IS_ON_INTERNAL;
					file = new File(mContext.getCacheDir().getAbsolutePath()
							+ "/" + MdotM_CACHE_FOLDER);
					MdotMLogger
							.i(this,
									"unable to create folder on sd card so creating on internal memory");
					file.mkdirs();
				}
			} else {
				MdotMLogger.i(this, "File created ");
			}
		}

		if (file.exists()) {
			clearCahe(file.listFiles());
		}
		MdotMLogger.i(this, "temp cache location is " + cacheLocation
				+ " file path " + file.getAbsolutePath());

	}

	private boolean checkAvailableMemory(int location) {
		long availableSize;
		if (location == MdotMCacheHandler.CACHE_IS_ON_INTERNAL) {
			availableSize = getAvailableInternalMemorySize();
		} else {
			availableSize = getAvailableExternalMemorySize();
		}
		if (availableSize > minMemorySize) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Takes all the file name from index table and compare it with the cached
	 * file. If there is mismatch (mostely happens if the android clears the
	 * cache in case of out of memory) then clear all index table and delete
	 * cached files. Also deletes index file name from cache index table if the
	 * same cached file doesn't exist.
	 * 
	 * @param filePath
	 */
	private boolean verifyCache(String filePath) {
		File cacheFolder = new File(filePath);
		if (!cacheFolder.exists()) {
			MdotMCacheHandler.getInstance(mContext).clearAllTable(mContext);
			cacheLocation = 0;
			return false;
		} else {
			String[] filesFromCacheTable = MdotMCacheHandler.getInstance(
					mContext).getAllFileFromCacheTable();
			String[] filesFromResourceTable = MdotMCacheHandler.getInstance(
					mContext).getAllFilesfromResourceTable();
			File[] files = cacheFolder.listFiles();
			if (filesFromCacheTable != null && files != null) {

				int fileCount = 0;
				if (filesFromCacheTable != null) {
					fileCount = fileCount + filesFromCacheTable.length;
				}
				if (filesFromResourceTable != null) {
					fileCount = fileCount + filesFromResourceTable.length;
				}
				int cachedFileCount = files.length;
				if (fileCount != cachedFileCount) {
					// if there is mismatch in the file count and cached file
					// count then clearing all table and all cached files
					MdotMCacheHandler.getInstance(mContext).clearAllTable(
							mContext);
					MdotMLogger.i(this, "file count missmatch clearing");
					clearCahe(files);
					cacheLocation = 0;
					return false;
				} else {
					String path;
					if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL) {
						path = mContext.getCacheDir().getAbsolutePath() + "/"
								+ MdotM_CACHE_FOLDER + "/";
					} else {
						path = Environment.getExternalStorageDirectory()
								.getAbsolutePath()
								+ "/"
								+ MdotM_CACHE_FOLDER
								+ "/";
					}
					if (filesFromCacheTable != null) {
						for (String file : filesFromCacheTable) {
							File cacheFile = new File(path + file);
							if (!cacheFile.exists()) {
								MdotMCacheHandler.getInstance(mContext)
										.deleteFileFromCacheTable(file);
								String[] resources = MdotMCacheHandler
										.getInstance(mContext)
										.getRelatedResources(file);
								if (resources != null) {
									for (String resource : resources) {
										try {
											MdotMCacheHandler
													.getInstance(mContext)
													.clearCachedResource(
															resource, mContext,
															cacheLocation, file);
										} catch (Exception e) {
											MdotMLogger
													.i(this,
															"Exception while delting the resources");
										}
									}
								}

							}

						}
					}
					// getting resources again
					filesFromResourceTable = MdotMCacheHandler.getInstance(
							mContext).getAllFilesfromResourceTable();
					if (filesFromResourceTable != null) {
						for (String res : filesFromResourceTable) {
							File cacheFile = new File(path + res);
							if (!cacheFile.exists()) {
								String[] parentFile = MdotMCacheHandler
										.getInstance(mContext)
										.getResourceParents(res);
								if (parentFile != null) {
									for (String parent : parentFile) {
										File f = new File(path + parent);
										if (f.exists()) {
											f.delete();
										}
										MdotMCacheHandler.getInstance(mContext)
												.deleteFileFromCacheTable(
														parent);
										String[] resources = MdotMCacheHandler
												.getInstance(mContext)
												.getRelatedResources(parent);
										for (String resource : resources) {
											try {
												MdotMCacheHandler.getInstance(
														mContext)
														.clearCachedResource(
																resource,
																mContext,
																cacheLocation,
																parent);
											} catch (Exception e) {
												MdotMLogger
														.i(this,
																"Exception while delting the resources");
											}
										}
									}
								}

							}

						}
					}
					return true;
				}

			} else {
				// either file fron cache table is null or files from cache is
				// null. so clearing all table and cache folder
				MdotMCacheHandler.getInstance(mContext).clearAllTable(mContext);
				clearCahe(files);
				cacheLocation = 0;
				return false;
			}

		}

	}

	private void clearCahe(File[] files) {
		MdotMLogger.i(this, "Clearing cached file");
		if (files != null) {
			for (File file : files) {
				if (file.isDirectory()) {
					clearCahe(file.listFiles());
					file.delete();
				} else {
					file.delete();
				}
			}
		}

	}

	/**
	 * returns the sd card state
	 * 
	 * @return true when sd card exist and is writable otherwise returns false
	 */
	private boolean checkSDCardState() {
		String state = Environment.getExternalStorageState();
		if (Environment.MEDIA_MOUNTED.equals(state)) {
			MdotMLogger.i(this, "SD card exist and writable");
			return true;
		} else if (Environment.MEDIA_MOUNTED_READ_ONLY.equals(state)) {
			MdotMLogger.i(this, "SD card exist but not writable");
			return false;
		} else {
			MdotMLogger.i(this, "SD card not exist");
			return false;
		}

	}

	private void setThreadPool() {
		executor = new ThreadPoolExecutor(CORE_POOL_SIZE, MAX_POOL_SIZE,
				KEEP_ALIVE_TIME_IN_SEC, TimeUnit.SECONDS, worksQueue);
	}

	public void setAdNetworkListener(MdotMNetworkListener listener) {
		this.networkListener = listener;
	}

	public void sendRequest(MdotMAdRequest params, long requestId) {
		MdotMLogger.i(this, "** Send request");
		if (params != null) {
			// mRequestId = requestId;
			MdotMNetworkManager.enableCaching = params.isEnableCaching();
			checkCacheStatus(params);
			try {
				if (executor != null) {
					executor.execute(new Requests(new MdotMRequestBuilder(
							mContext).buildRequest(params), params, requestId));

				}
			} catch (RejectedExecutionException e) {
				MdotMLogger.i(this, "**Reject Exception");
				Message msg = new Message();
				msg.obj = requestId;
				mHandler.sendMessage(msg);
			} catch (Exception e) {
				MdotMLogger.e(this, e.getMessage());
				Message msg = new Message();
				msg.obj = requestId;
				mHandler.sendMessage(msg);
			}
		}
	}

	class Requests implements Runnable {

		HttpUriRequest httpRequest;
		MdotMAdRequest adRequest;
		long requestID;

		public Requests(HttpUriRequest httpUriRequest, MdotMAdRequest params,
				long requestId) {
			this.httpRequest = httpUriRequest;
			this.adRequest = params;
			requestID = requestId;
		}

		public void run() {
			HttpResponse response = null;
			HttpParams httpParameters = new BasicHttpParams();
			// Set the timeout in milliseconds until a connection is
			// established.
			// The default value is zero, that means the timeout is not used.
			int timeoutConnection = 10000;
			HttpConnectionParams.setConnectionTimeout(httpParameters,
					timeoutConnection);
			// Set the default socket timeout (SO_TIMEOUT)
			// in milliseconds which is the timeout for waiting for data.
			int timeoutSocket = 10000;
			HttpConnectionParams.setSoTimeout(httpParameters, timeoutSocket);
			HttpClient httpClient = new DefaultHttpClient(httpParameters);
			try {
				response = httpClient.execute(httpRequest);
				MdotMLogger.d(this, "The request is being sent");
				int responseCode = response.getStatusLine().getStatusCode();
				// MdotMLogger.d(this, "The response code is:" + responseCode);
				if (responseCode == HTTP_RESPONSE_OK) {
					MdotMAdResponse adResponse = new MdotMJSONParser()
							.parseResponse(response.getEntity().getContent());
					if (adResponse != null && adResponse.getStatus() != 0) {
						if (this.adRequest.isLoadInterstitial()) {
							adResponse.setRequestForBannerAd(false);
						} else {
							adResponse.setRequestForBannerAd(true);
						}
						// setting requested ad size in the response for later
						// use to set the adsize layout dynamically
						String adSize = this.adRequest.getAdSize();
						if (adSize != null) {
							String[] sizes = adSize.split(",");
							MdotMLogger.i(this, "Ad width and height "
									+ Integer.parseInt(sizes[0]) + " "
									+ Integer.parseInt(sizes[1]));
							adResponse.setWidth(Integer.parseInt(sizes[0]));
							adResponse.setHeight(Integer.parseInt(sizes[1]));
						}

						new AdContentDownloader().downloadResources(adResponse,
								cacheLocation, mContext,
								adRequest.isEnableCaching());

						networkListener.finishedAdParsing(adResponse);
					} else {
						networkListener.finishedAdParsing(adResponse);
					}
				} else {
					networkListener.finishedAdParsing(null);
				}
			} catch (ClientProtocolException e) {
				MdotMLogger
						.e(this, "ClientProtocolException " + e.getMessage());
				networkListener.finishedAdParsing(null);
			} catch (IOException e) {
				MdotMLogger.e(this, "IOException " + e.getMessage());
				networkListener.finishedAdParsing(null);
			}
			Message msg = new Message();
			msg.obj = requestID;
			mHandler.sendMessage(msg);
		}
	}

	private void manageCacheSize() {
		int cacheSize;

		int maxCacheSize;
		if (cacheLocation != 0) {
			if (cacheLocation == MdotMCacheHandler.CACHE_IS_ON_INTERNAL) {
				cacheSize = getSize(new File(mContext.getCacheDir()
						.getAbsolutePath() + "/" + MdotM_CACHE_FOLDER), 0);
				maxCacheSize = adCacheSizeInternal;

				MdotMLogger.i(this, "cache size is " + cacheSize + " path "
						+ mContext.getCacheDir().getAbsolutePath() + "/"
						+ MdotM_CACHE_FOLDER);
			} else {
				cacheSize = getSize(new File(Environment
						.getExternalStorageDirectory().getAbsolutePath()
						+ "/"
						+ MdotM_CACHE_FOLDER), 0);
				maxCacheSize = adCacheSizeSDCard;

				MdotMLogger.i(this, "cache size is "
						+ cacheSize
						+ " path "
						+ Environment.getExternalStorageDirectory()
								.getAbsolutePath() + "/" + MdotM_CACHE_FOLDER
						+ " max cache size" + maxCacheSize);
			}
			if (cacheSize > maxCacheSize) {
				if (deleteLeastRecentlyUsedCache())
					manageCacheSize();
			}
		}
	}

	private boolean deleteLeastRecentlyUsedCache() {
		boolean canDelete = MdotMCacheHandler.getInstance(mContext)
				.deleteCache(mContext, cacheLocation);
		return canDelete;
	}

	private int getSize(File directory, int sizeInBytes) {
		if (directory == null)
			return sizeInBytes;
		if (!directory.exists())
			return sizeInBytes;
		if (directory.isDirectory()) {
			String[] list = directory.list();
			// Some JVMs return null for File.list() when the
			// directory is empty.
			if (list != null) {
				for (int i = 0; i < list.length; i++) {
					File entry = new File(directory, list[i]);
					if (entry.isDirectory()) {
						sizeInBytes = getSize(entry, sizeInBytes);
					} else {
						sizeInBytes += entry.length();
					}
				}
			}
		}
		return sizeInBytes;
	}

	private long getAvailableInternalMemorySize() {
		File path = Environment.getDownloadCacheDirectory();
		StatFs stat = new StatFs(path.getPath());
		long blockSize = stat.getBlockSize();
		long availableBlocks = stat.getAvailableBlocks();
		return (availableBlocks * blockSize);
	}

	private long getAvailableExternalMemorySize() {
		if (isSDCardExist) {
			File path = Environment.getExternalStorageDirectory();
			StatFs stat = new StatFs(path.getPath());
			long blockSize = stat.getBlockSize();
			long availableBlocks = stat.getAvailableBlocks();
			return (availableBlocks * blockSize);
		} else {
			return 0;
		}
	}

}
