package com.mdotm.android.http;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.content.Context;
import android.os.Handler;
import android.os.Message;

import com.mdotm.android.listener.MdotMNetworkListener;
import com.mdotm.android.model.MdotMAdRequest;
import com.mdotm.android.utils.MdotMLogger;

public class MdotMRequestQueueClient {
	private static MdotMRequestQueueClient instance = null;
	private static Handler handler;
	private Map<Long, MdotMNetworkListener> mapNetworkListenerQueue = new HashMap<Long, MdotMNetworkListener>();
	private Map<Long, MdotMAdRequest> mapRequestQueue = new HashMap<Long, MdotMAdRequest>();
	private List<Long> listRequestId = new ArrayList<Long>();
	private List<Long> listActiveRequest = new ArrayList<Long>();

	MdotMNetworkManager mAdManager;
	Context mContext;
	private boolean applicationHasFocus;

	public static MdotMRequestQueueClient getInstance(Context context) {
		if (null == instance) {
			instance = new MdotMRequestQueueClient(context);
		}
		return instance;
	}

	private MdotMRequestQueueClient(Context context) {
		mContext = context;
		if (handler == null) {
			MdotMLogger.e(this, "Creating new handler");
			handler = new Handler() {

				@Override
				public void handleMessage(Message msg) {
					try {
						MdotMLogger.i(this, "Handling the message");
						if (msg != null) {
							MdotMLogger.d(this, " message id " + msg.obj);
							if (listActiveRequest.contains(msg.obj)) {
								MdotMLogger.i(this, "contains requested id ");
								listActiveRequest.remove(msg.obj);
								mapRequestQueue.remove(msg.obj);
								mapNetworkListenerQueue.remove(msg.obj);
								listRequestId.remove(msg.obj);

								if (listRequestId.size() > 0) {
									if (isApplicationHasFocus()) {
										Long reqId = listRequestId.get(0);
										MdotMLogger.d(this,
												"mapRequestQueue size > 0");
										sendNextRequest(
												mapRequestQueue.get(reqId),
												mapNetworkListenerQueue
														.get(reqId), reqId);
									} else {
										for (int i = 0; i < listRequestId
												.size(); i++) {
											Long reqId = listRequestId.get(i);
											MdotMAdRequest req = mapRequestQueue
													.get(reqId);
											if (!req.isLoadInterstitial()) {
												mapRequestQueue.remove(reqId);
												mapNetworkListenerQueue
														.remove(reqId);
												listRequestId.remove(reqId);

											}
										}
										if (listRequestId.size() > 0) {
											MdotMLogger
													.i(this,
															"still has some interstitial requests so processing");
											Long reqId = listRequestId.get(0);
											sendNextRequest(
													mapRequestQueue.get(reqId),
													mapNetworkListenerQueue
															.get(reqId), reqId);
										}
									}
								} else if (listActiveRequest.size() == 0) {
									listActiveRequest.clear();
									mapRequestQueue.clear();
									mapNetworkListenerQueue.clear();
									listRequestId.clear();

								}

							}

						}
						super.handleMessage(msg);
					} catch (Exception e) {
						try {
							listActiveRequest.clear();
							mapRequestQueue.clear();
							mapNetworkListenerQueue.clear();
							listRequestId.clear();
						} catch (Exception e1) {

						}
					}
				}

			};
		}
	}

	protected void sendNextRequest(MdotMAdRequest mdotMAdRequest,
			MdotMNetworkListener mdotMNetworkListener, long requestId) {
		MdotMLogger.i(this, "** sending req for  " + requestId);
		mAdManager = MdotMNetworkManager.getInstance(mContext, handler);
		mAdManager.setAdNetworkListener(mdotMNetworkListener);
		mAdManager.sendRequest(mdotMAdRequest, requestId);
		listActiveRequest.add(requestId);
	}

	public void addRequest(MdotMAdRequest request, MdotMNetworkListener listener) {
		long requestId = generateId();
		MdotMLogger.i(this, "** added to queue " + requestId);
		listRequestId.add(requestId);

		mapNetworkListenerQueue.put(requestId, listener);
		mapRequestQueue.put(requestId, request);
		if (listActiveRequest.size() == 0) {
			sendNextRequest(request, listener, requestId);

		}
	}

	private long generateId() {
		return System.currentTimeMillis();
	}

	public boolean isApplicationHasFocus() {
		return applicationHasFocus;
	}

	public void setApplicationHasFocus(boolean applicationHasFocus) {
		this.applicationHasFocus = applicationHasFocus;
	}

}
