package com.mdotm.android.utils;

/**
 * @author Bheema,Meghana
 * 
 */

import java.net.URLEncoder;

import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpUriRequest;

import android.content.Context;
import android.provider.Settings.Secure;
import android.telephony.TelephonyManager;

import com.mdotm.android.model.MdotMAdRequest;

public class MdotMRequestBuilder {
	final String PARTNER_KEY = "mdotm";
	final String API_KEY = "mdotmsdk2";
	final String APP_VERSION = "2.0.2";
	private Context mContext;

	public MdotMRequestBuilder(Context context) {
		this.mContext = context;
	}

	/**
	 * Builds a request to fetch the ads
	 * 
	 * @param request
	 * @return
	 */
	public HttpUriRequest buildRequest(MdotMAdRequest request) {
		String adSize = request.getAdSize();

		StringBuffer url = new StringBuffer(MdotMConfiguration.MDOTM_BASE_URL);

		url.append("partnerkey=" + PARTNER_KEY);
		url.append("&");
		url.append("apikey=" + API_KEY);
		// url.append("&clientip=68.45.124.20");
		// url.append("&clientip=199.180.254.169");
		url.append("&");
		url.append("appkey=" + request.getAppKey().trim());
		url.append("&");
		url.append("ua="
				+ URLEncoder.encode(MdotMUtils.getUtilsInstance().getUserAgent(
						mContext)));
		url.append("&");
		if (adSize != null && adSize.length() > 0) {
			String[] sizes = adSize.split(",");
			url.append("width=" + sizes[0]);
			MdotMLogger.d(this, "The width is:" + sizes[0]);
			url.append("&");
			url.append("height=" + sizes[1]);
			MdotMLogger.d(this, "The height is:" + sizes[1]);
			url.append("&");
		}
		url.append("fmt=" + "json");
		url.append("&");
		url.append("v=" + APP_VERSION);
		url.append("&");
		if (MdotMUtils.getUtilsInstance().getBandWidth(mContext) != null) {
			url.append("bandwidth="
					+ MdotMUtils.getUtilsInstance().getBandWidth(mContext));
			MdotMLogger.i(this, "The bandwidth is:"
					+ MdotMUtils.getUtilsInstance().getBandWidth(mContext));
			url.append("&");
		}
		if (request.isLoadInterstitial()) {
			url.append("interstitial_ad=" + "true");
			url.append("&");
		} else {
			url.append("interstitial_ad=" + "false");
			url.append("&");
		}
		url.append("test=" + request.isTestMode());
		try {
			String deviceId = getImei();
			url.append("&");
			url.append("deviceid=" + deviceId);
			MdotMLogger.i(this, "imei device id " + deviceId);
		} catch (Exception e) {
			MdotMLogger.i(this,
					"Exception while reading imei " + e.getMessage());
		}
		try {
			String deviceId = getAndroidId();
			MdotMLogger.i(this, "android device id " + deviceId);
			url.append("&");
			url.append("aid=" + deviceId);
		} catch (Exception e) {
			// TODO: handle exception
		}

		MdotMLogger.e(this, "The url is:" + url.toString()
				+ "  the adresponse is:" + request.isLoadInterstitial());
		// String testURL =
		// "http://ads.mdotm.com/ads/feed.php?partnerkey=mdotm&apikey=mdotmsdk2&appkey=353f274697b595983e6fceb0cbf7d5d3&ua=Mozilla%2F5.0+%28Linux%3B+U%3B+Android+4.0.4%3B+el-gr%3B+PMP3570C+Build%2FPMP3570C_20130118_v1.0.7%29+AppleWebKit%2F534.30+%28KHTML%2C+like+Gecko%29+Version%2F4.0+Safari%2F534.30&width=800&height=444&fmt=json&v=2.0.2&interstitial_ad=true&test=0&deviceid=\nPMP100130246006&aid=da193c85680e622c";
		// MdotMLogger.i(this,
		// "removed white spaces "+url.toString().replaceAll("\\s", ""));
		HttpGet httpGetRequest = new HttpGet(url.toString());
		return httpGetRequest;

	}

	/**
	 * Returns the device id
	 * 
	 * @return
	 */
	private String getAndroidId() {
		String device = Secure.getString(mContext.getContentResolver(),
				Secure.ANDROID_ID);
		device = device.replaceAll("\\s", "");
		return device;
	}

	/**
	 * Returns IMEI number
	 */
	private String getImei() {
		String device = ((TelephonyManager) mContext
				.getSystemService(Context.TELEPHONY_SERVICE)).getDeviceId();
		device = device.replaceAll("\\s", "");

		return device;
	}

}
