package com.mdotm.android.utils;

import java.io.IOException;
import java.net.MalformedURLException;
import java.security.NoSuchAlgorithmException;
import java.util.Locale;

import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.StatusLine;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.HttpConnectionParams;

import android.annotation.TargetApi;
import android.content.Context;
import android.graphics.Point;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.AsyncTask;
import android.os.Build;
import android.telephony.TelephonyManager;
import android.view.Display;
import android.view.WindowManager;
import android.webkit.WebView;

public class MdotMUtils {
	public static int AD_ICON_WITH_TEXT_MESSAGE = 2;
	public static int AD_WITH_BANNER_OR_XHTML_OR_VIDEO = 1;
	public static int AD_RECEVIED = 1;

	private static MdotMUtils _mdotmUtilsInstance;

	public static MdotMUtils getUtilsInstance() {
		if (_mdotmUtilsInstance == null) {
			_mdotmUtilsInstance = new MdotMUtils();
		}
		return _mdotmUtilsInstance;
	}

	public String getDeviceId(Context context, String deviceId)
			throws NoSuchAlgorithmException {
		if ((deviceId == null) || (deviceId.length() < 32)) {
			TelephonyManager telephonyManager = (TelephonyManager) context
					.getSystemService(Context.TELEPHONY_SERVICE);
			deviceId = telephonyManager.getDeviceId();
		}
		return deviceId;
	}

	/**
	 * Get the useragent of the device
	 * 
	 * @return
	 */
	public String getUserAgent(Context context) {
		String userAgent = null;
		try {
			userAgent = new WebView(context).getSettings().getUserAgentString();
		} catch (Exception e) {
			MdotMLogger.e(this, e.getMessage());
			userAgent = getUserAgentManually();
		}
		return userAgent;
	}

	private String getUserAgentManually() {
		StringBuffer arg = new StringBuffer();

		// Android version
		final String version = Build.VERSION.RELEASE;
		if (version.length() > 0) {
			arg.append(version);
		} else {
			// default to "1.0"
			arg.append("1.0");
		}
		arg.append("; ");

		// Initialize the mobile user agent with the default locale.
		final Locale l = Locale.getDefault();
		final String language = l.getLanguage();
		if (language != null) {

			arg.append(language.toLowerCase());
			final String country = l.getCountry();

			if (country != null) {

				arg.append("-");
				arg.append(country.toLowerCase());
			}

		} else {
			// default to "en"
			arg.append("en");
		}

		// Add the device model name and Android build ID.
		final String model = Build.MODEL;
		if (model.length() > 0) {

			arg.append("; ");
			arg.append(model);
		}

		final String id = Build.ID;
		if (id.length() > 0) {

			arg.append(" Build/");
			arg.append(id);
		}

		final String base = "Mozilla/5.0 (Linux; U; Android %s) AppleWebKit/533.1 (KHTML, like Gecko) Version/4.0 Mobile Safari/533.1";
		return String.format(base, arg);

	}

	@TargetApi(Build.VERSION_CODES.HONEYCOMB_MR2)
	public String getDisplaySize(Context mContext) {

		int Measuredwidth = 0;
		int Measuredheight = 0;
		Point size = new Point();
		WindowManager windowManager = (WindowManager) mContext
				.getSystemService(Context.WINDOW_SERVICE);
		try {
			if (Build.VERSION.SDK_INT >= 13) {
				MdotMLogger.i(this, "Getting conditional display size");
				windowManager.getDefaultDisplay().getSize(size);
				Measuredwidth = size.x;
				Measuredheight = size.y;
			} else {
				MdotMLogger.i(this, "Default display");
				Display d = windowManager.getDefaultDisplay();
				Measuredwidth = d.getWidth();
				Measuredheight = d.getHeight();
			}
		} catch (Exception e) {
			e.printStackTrace();
			// On rooted phone "Build.VERSION.SDK_INT" returns null. So getting
			// display size using default methods
			MdotMLogger.i(this, "Default display");
			Display d = windowManager.getDefaultDisplay();
			Measuredwidth = d.getWidth();
			Measuredheight = d.getHeight();
		}

		String displaySize = Measuredwidth + "," + Measuredheight;
		MdotMLogger.i(this, "SCREEN DISPLAY SIZE " + displaySize);
		return displaySize;
	}

	/**
	 * Get the bandwidth of the active network
	 * 
	 * @return
	 */
	public String getBandWidth(Context context) {
		ConnectivityManager cm = (ConnectivityManager) context
				.getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo info = cm.getActiveNetworkInfo();
		int type = 0;
		int subType;
		if (info != null && info.isConnected()) {
			type = info.getType();
			subType = info.getSubtype();
			if (type == ConnectivityManager.TYPE_WIFI) {
				try {
					WifiManager wifiManager = (WifiManager) context
							.getSystemService(Context.WIFI_SERVICE);
					WifiInfo wifiInfo = wifiManager.getConnectionInfo();
					if (wifiInfo != null) {
						Integer linkSpeed = wifiInfo.getLinkSpeed(); // measured
						MdotMLogger.i(this, "Wifi bandwidth " + linkSpeed);
						// using
						return String.valueOf(linkSpeed) + "000"; // WifiInfo.LINK_SPEED_UNITS
					}
				} catch (Exception e) {
					// e.printStackTrace();
				}
				return null;
			} else if (type == ConnectivityManager.TYPE_MOBILE) {
				switch (subType) {
				case TelephonyManager.NETWORK_TYPE_1xRTT:
					return "50"; // ~ 50-100 kbps
				case TelephonyManager.NETWORK_TYPE_CDMA:
					return "14"; // ~ 14-64 kbps
				case TelephonyManager.NETWORK_TYPE_EDGE:
					return "50"; // ~ 50-100 kbps
				case TelephonyManager.NETWORK_TYPE_EVDO_0:
					return "400"; // ~ 400-1000 kbps
				case TelephonyManager.NETWORK_TYPE_EVDO_A:
					return "600"; // ~ 600-1400 kbps
				case TelephonyManager.NETWORK_TYPE_GPRS:
					return "100"; // ~ 100 kbps
				case TelephonyManager.NETWORK_TYPE_HSDPA:
					return "2000"; // ~ 2-14 Mbps
				case TelephonyManager.NETWORK_TYPE_HSPA:
					return "700"; // ~ 700-1700 kbps
				case TelephonyManager.NETWORK_TYPE_HSUPA:
					return "1024"; // ~ 1-23 Mbps
				case TelephonyManager.NETWORK_TYPE_UMTS:
					return "400"; // ~ 400-7000 kbps
					// NOT AVAILABLE YET IN API LEVEL 7
				case TelephonyManager.NETWORK_TYPE_EHRPD:
					return "1024"; // ~ 1-2 Mbps
				case TelephonyManager.NETWORK_TYPE_EVDO_B:
					return "5120"; // ~ 5 Mbps
				case TelephonyManager.NETWORK_TYPE_HSPAP:
					return "10240"; // ~ 10-20 Mbps
				case TelephonyManager.NETWORK_TYPE_IDEN:
					return "25600"; // ~25 kbps
				case TelephonyManager.NETWORK_TYPE_LTE:
					return "10240"; // ~ 10+ Mbps
					// Unknown
				case TelephonyManager.NETWORK_TYPE_UNKNOWN:
					return null;
				default:
					return null;
				}
			}
		} else {
			return null;
		}
		return null;
	}

	public void reportImpression(String impPixel, Context context) {
		new MyAsynchTask(context).execute(impPixel);
	}

	class MyAsynchTask extends AsyncTask<String, Void, Void> {
		Context context;

		public MyAsynchTask(Context con) {
			context = con;
		}

		@Override
		protected Void doInBackground(String... params) {
			String impPixel = params[0];
			MdotMLogger.i(this, "Reporting impression " + impPixel);
			if (impPixel != null && impPixel.trim().length() > 0) {
				try {
					if (isNetworkAvailable(context)) {
						HttpClient httpClient = new DefaultHttpClient();
						httpClient.getParams().setParameter(
								HttpConnectionParams.CONNECTION_TIMEOUT,
								new Integer(5000));
						HttpResponse response = httpClient.execute(new HttpGet(
								impPixel));
						if (response != null) {
							StatusLine statusLine = response.getStatusLine();
							if (statusLine.getStatusCode() == HttpStatus.SC_OK) {
								MdotMLogger.i(this, "impression successfull "
										+ statusLine.getStatusCode() + " "
										+ statusLine.getReasonPhrase());
								response.getEntity().getContent().close();
							} else {
								MdotMLogger
										.i(this, "impression unsuccessfull ");
								// Closes the connection.
								response.getEntity().getContent().close();
							}
						} else {
							MdotMLogger.e(this,
									"Impression reporting response is NULL");
						}

					}
				} catch (MalformedURLException e) {
					MdotMLogger.e(this,
							"MalformedURLException exception while reporting impression "
									+ e.getMessage());
					e.printStackTrace();
				} catch (IOException e) {
					MdotMLogger.e(this,
							"IOException exception while reporting impression "
									+ e.getMessage());
					e.printStackTrace();
				} catch (Exception e) {
					MdotMLogger.e(this, "Exception while recording impression "
							+ e.getMessage());
					e.printStackTrace();
				}
			} else {
				MdotMLogger.i(this, "imp pixel url is null");
			}
			return null;

		}

	}

	private boolean isNetworkAvailable(Context context) {
		ConnectivityManager cm = (ConnectivityManager) context
				.getApplicationContext().getSystemService(
						Context.CONNECTIVITY_SERVICE);
		NetworkInfo ni = cm.getActiveNetworkInfo();
		if (ni != null && ni.isConnected()) {
			return true;
		} else {
			return false;
		}
	}
}
