package com.mdotm.android.view;

import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.net.Uri;
import android.os.Handler;
import android.view.Gravity;
import android.view.View;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;

import com.mdotm.android.listener.MdotMAdActionListener;
import com.mdotm.android.model.MdotMAdResponse;
import com.mdotm.android.utils.MdotMLogger;

public class HtmlView extends RelativeLayout {
	private MdotMAdActionListener mActionListener;
	private MdotMAdResponse mAdResponse;
	private ProgressBar activityIndicator;
	private boolean adSelectionInProgress;
	private WebView webView = null;
	private boolean isAdClicked = false;
	Handler handler;

	public HtmlView(Context context, MdotMAdResponse adResponse,
			MdotMAdActionListener listener) {
		super(context);
		handler = new Handler();
		mActionListener = listener;
		mAdResponse = adResponse;
		RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(
				mAdResponse.getWidth(), mAdResponse.getHeight());
		params.addRule(RelativeLayout.CENTER_IN_PARENT);
		this.setLayoutParams(params);
		this.setGravity(Gravity.CENTER);
		activityIndicator = new ProgressBar(getContext());
		activityIndicator.setIndeterminate(false);
		LayoutParams activityParams = new LayoutParams(40, 40);
		activityIndicator.setLayoutParams(activityParams);
		activityIndicator.setVisibility(INVISIBLE);
		activityIndicator.setMinimumHeight(8);
		activityIndicator.setMinimumWidth(8);
		activityIndicator.setMax(100);
		if (activityIndicator != null) {
			activityIndicator.setId(3); // for alignment.
			activityParams.addRule(CENTER_IN_PARENT);
		}
		addView(activityIndicator);
		webView = new WebView(context);
		// webView.setVisibility(View.INVISIBLE);
		webView.getSettings().setJavaScriptEnabled(true);
		webView.setWebViewClient(new WebViewClient() {
			boolean timeout;
			Runnable run = new Runnable() {
				public void run() {

					if (timeout) {
						MdotMLogger.e(this, "timeout loading landing url");
						webView.stopLoading();
						adNetworkCompleted();
						timeout = false;
					}
				}
			};

			@Override
			public void onPageStarted(WebView view, String url, Bitmap favicon) {
				MdotMLogger.i(this, "started Loading url " + url);
				if (isAdSelectionInProgress()) {
					timeout = true;
					handler.removeCallbacks(run);
					handler.postDelayed(run, 15000);
				}
				super.onPageStarted(view, url, favicon);
			}

			@Override
			public boolean shouldOverrideUrlLoading(WebView view, String url) {
				MdotMLogger.i(this, "returned url " + url);
				if (url != null && url.startsWith("market://")) {
					view.getContext().startActivity(
							new Intent(Intent.ACTION_VIEW, Uri.parse(url)));
					mActionListener.leaveApplication();

					// Lets reset clickable and progress flag
					timeout = false;
					adNetworkCompleted();
					return true;
				} else {
					MdotMLogger.i(this, "loading other url " + url);
					return super.shouldOverrideUrlLoading(view, url);
				}
			}

			@Override
			public void onPageFinished(WebView view, String url) {
				super.onPageFinished(view, url);
				timeout = false;
				MdotMLogger.i(this, "Click page finished " + url);
				adNetworkCompleted();
			}

			@Override
			public void onReceivedError(WebView view, int errorCode,
					String description, String failingUrl) {
				super.onReceivedError(view, errorCode, description, failingUrl);
				timeout = false;
				if (isAdSelectionInProgress()) {
					MdotMLogger.e(this, "Unable to load landing url : "
							+ description);
				} else {
					MdotMLogger.e(this, "Unable to report impression : "
							+ description);
				}
				// Lets reset clickable and progress flag
				adNetworkCompleted();
			}
		});
		initView(context);
	}

	private void initView(final Context context) {
		// RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(
		// RelativeLayout.LayoutParams.WRAP_CONTENT,
		// RelativeLayout.LayoutParams.WRAP_CONTENT);
		// this.setLayoutParams(params);
		MdotMWebView webView = new MdotMWebView(context, mAdResponse,
				mActionListener);
		this.addView(webView);
		// adding extra layout to handle the click listener
		RelativeLayout clickLayout = new RelativeLayout(context);
		RelativeLayout.LayoutParams clickparams = new RelativeLayout.LayoutParams(
				mAdResponse.getWidth(), mAdResponse.getHeight());
		clickparams.addRule(RelativeLayout.CENTER_IN_PARENT);
		clickLayout.setLayoutParams(clickparams);
		clickLayout.setClickable(true);
		clickLayout.setEnabled(true);
		clickLayout.setBackgroundColor(Color.TRANSPARENT);
		clickLayout.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				MdotMLogger.i(this, "Clicked on click layout ");
				if (!isAdClicked) {
					isAdClicked = true;
					clicked(context);
				}
			}
		});

		this.addView(clickLayout);
	}

	private void setAdSelectionInProgress(boolean isProgress) {
		this.adSelectionInProgress = isProgress;
	}

	private boolean isAdSelectionInProgress() {
		return adSelectionInProgress;
	}

	private void adNetworkCompleted() {
		MdotMLogger.i(this, "On ad network completed");
		setAdSelectionInProgress(false);
		isAdClicked = false;
		hideActivityIndicator();
	}

	private void showActivityIndicator() {
		post(new Thread() {
			public void run() {
				if (activityIndicator != null) {
					activityIndicator.bringToFront();
					activityIndicator.setVisibility(VISIBLE);
				}
			}
		});
	}

	private void hideActivityIndicator() {
		post(new Thread() {
			public void run() {
				if (activityIndicator != null) {
					activityIndicator.setVisibility(INVISIBLE);
				}
			}
		});
	}

	protected void clicked(Context context) {
		MdotMLogger.i(this, "  Selected to clicked  ");

		if (mAdResponse.getLandingUrl() != null) {

			if (!isAdSelectionInProgress()) {
				final String clickedUrl = mAdResponse.getLandingUrl();
				setAdSelectionInProgress(true);
				showActivityIndicator();
				new Thread() {
					public void run() {
						mActionListener.adClicked();
						MdotMLogger
								.i(this,
										"Launch type is "
												+ mAdResponse.getLaunchType());
						if (mAdResponse.getLaunchType() == 2) {

							Intent i = new Intent(Intent.ACTION_VIEW,
									Uri.parse(clickedUrl));
							i.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

							try {

								getContext().startActivity(i);

							} catch (Exception e) {

								MdotMLogger.i(this,
										"Could not open browser on ad click to "
												+ e);
							}

							MdotMLogger.i(this, "Leaving app");
							mActionListener.leaveApplication();
							adNetworkCompleted();

						} else {

							handler.post(new Runnable() {

								public void run() {
									webView.loadUrl(clickedUrl);
								}
							});
						}

					}
				}.start();
			} else {
				MdotMLogger.i(this, "ad selection under progress");
			}
		} else {
			MdotMLogger.i(this, "selected ad is null");
		}
	}

}
