package com.mdotm.android.view;

import java.io.IOException;
import java.io.InputStream;
import java.lang.ref.WeakReference;
import java.net.URL;
import java.net.URLConnection;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;

import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Handler;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.ImageView;
import android.widget.ImageView.ScaleType;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;

import com.mdotm.android.listener.MdotMAdActionListener;
import com.mdotm.android.model.MdotMAdResponse;
import com.mdotm.android.utils.MdotMLogger;
import com.mdotm.android.utils.MdotMUtils;

/**
 * @author Bheema
 * 
 */
class InterstitialImageView extends RelativeLayout {

	private boolean adSelectionInProgress;
	private String adLandingUrl;
	private int launchType;
	public boolean adLoadSuccess;
	private MdotMAdActionListener mAdClickListener;
	private ProgressBar activityIndicator;
	private static final int HIGHLIGHT_COLOR = 0xFFFFB400; // Android yellow

	private static final int HIGHLIGHT_BACKGROUND_COLOR = 0xFFEE7F27; // android
	// orange
	private int backgroundColor = Color.BLACK;

	private Drawable adSelectedBackground;
	private Drawable defaultBackground;
	private WebView webView = null;
	private boolean isAdClicked = false;
	Handler handler;

	/**
	 * Constructs an advertisement view from a layout XML file.
	 * 
	 * @param context
	 *            is the application's context.
	 * @param attrs
	 *            are attributes set in the XML layout for this view.
	 * @param defStyle
	 *            is the theme ID to apply to this view.
	 * 
	 * @see android.view.View#View(Context, AttributeSet, int)
	 */
	public InterstitialImageView(Context context, AttributeSet attrs,
			int defStyle) {
		super(context, attrs, defStyle);
	}

	/**
	 * Constructs an advertisement view from a layout XML file.
	 * 
	 * @param context
	 *            is the application's context.
	 * @param attrs
	 *            are attributes set in the XML layout for this view.
	 * 
	 * @see android.view.View#View(android.content.Context,
	 *      android.util.AttributeSet)
	 */
	public InterstitialImageView(Context context, AttributeSet attrs) {
		this(context, attrs, 0);
	}

	public InterstitialImageView(Context context, MdotMAdResponse adResponse,
			MdotMAdActionListener listener) {
		super(context);
		handler = new Handler();
		launchType = adResponse.getLaunchType();
		webView = new WebView(context);
		webView.getSettings().setJavaScriptEnabled(true);
		webView.setWebViewClient(new WebViewClient() {
			boolean timeout;
			Runnable run = new Runnable() {
				public void run() {

					if (timeout) {
						MdotMLogger.e(this, "timeout loading landing url");
						webView.stopLoading();
						adNetworkCompleted();
						timeout = false;
					}
				}
			};

			@Override
			public void onPageStarted(WebView view, String url, Bitmap favicon) {
				MdotMLogger.i(this, "started Loading url " + url);
				if (isAdSelectionInProgress()) {
					timeout = true;

					handler.removeCallbacks(run);
					handler.postDelayed(run, 15000);
				}
				super.onPageStarted(view, url, favicon);
			}

			@Override
			public boolean shouldOverrideUrlLoading(WebView view, String url) {
				MdotMLogger.i(this, "returned url " + url);
				if (url != null && url.startsWith("market://")) {
					view.getContext().startActivity(
							new Intent(Intent.ACTION_VIEW, Uri.parse(url)));
					mAdClickListener.leaveApplication();

					// Lets reset clickable and progress flag
					adNetworkCompleted();
					timeout = false;
					return true;
				} else {
					MdotMLogger.i(this, "loading other url " + url);
					return super.shouldOverrideUrlLoading(view, url);
				}
			}

			@Override
			public void onPageFinished(WebView view, String url) {
				super.onPageFinished(view, url);
				timeout = false;
				MdotMLogger.i(this, "Click page finished " + url);
				adNetworkCompleted();
			}

			@Override
			public void onReceivedError(WebView view, int errorCode,
					String description, String failingUrl) {
				super.onReceivedError(view, errorCode, description, failingUrl);
				timeout = false;
				if (isAdSelectionInProgress()) {
					MdotMLogger.e(this, "Unable to load landing url : "
							+ description);
				} else {
					MdotMLogger.e(this, "Unable to report impression : "
							+ description);
				}
				// Lets reset clickable and progress flag
				adNetworkCompleted();
			}
		});

		RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(
				RelativeLayout.LayoutParams.WRAP_CONTENT,
				RelativeLayout.LayoutParams.WRAP_CONTENT);
		params.addRule(RelativeLayout.CENTER_IN_PARENT);
		this.setLayoutParams(params);
		mAdClickListener = listener;
		setFocusable(true);
		setDescendantFocusability(ViewGroup.FOCUS_AFTER_DESCENDANTS);
		setClickable(true);
		this.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				if (!isAdClicked) {
					isAdClicked = true;
					setClickable(false);
					clicked(InterstitialImageView.this.getContext());
				}

			}
		});
		setAdSelectionInProgress(false);
		try {
			initializeAdView(adResponse, context);
		} catch (Exception e) {
			MdotMLogger.e(this, "Exception initializing interstitial adview");
			adLoadSuccess = false;
			return;
		}
	}

	private void initializeAdView(MdotMAdResponse adResponse, Context context)
			throws Exception {
		if (adResponse != null) {
			this.adLandingUrl = adResponse.getLandingUrl();
			// The user can interact with the ad by clicking on it.
			setFocusable(true);
			setClickable(true);

			// Bitmap icon;
			WeakReference<Bitmap> bannerBitmap;
			if (adResponse.isCachedLocally()) {

				// icon = BitmapFactory.decodeFile(adResponse.getImageUrl());
				bannerBitmap = new WeakReference<Bitmap>(
						BitmapFactory.decodeFile(adResponse.getImageUrl()));

			} else {
				// icon = fetchImage(adResponse.getImageUrl(), false);
				bannerBitmap = new WeakReference<Bitmap>(fetchImage(
						adResponse.getImageUrl(), false));
			}

			MdotMLogger.i(this, "Image is " + adResponse.getImageUrl());
			if (bannerBitmap.get() != null) {

				activityIndicator = new ProgressBar(getContext());
				activityIndicator.setIndeterminate(false);
				LayoutParams activityParams = new LayoutParams(40, 40);
				activityIndicator.setLayoutParams(activityParams);
				activityIndicator.setVisibility(INVISIBLE);
				activityIndicator.setMinimumHeight(30);
				activityIndicator.setMinimumWidth(30);
				activityIndicator.setMax(100);
				// activityIndicator.setBackgroundResource(Color.TRANSPARENT);

				// set the attributes.
				if (adResponse.getAdType() == MdotMUtils.AD_WITH_BANNER_OR_XHTML_OR_VIDEO) {
					createAdWithBannerView(bannerBitmap.get(), adResponse);
					if (activityIndicator != null) {
						activityIndicator.setId(2); // for alignment.
						activityParams.addRule(CENTER_IN_PARENT);
					}

				} else {
					adLoadSuccess = false;
					MdotMLogger
							.e(this,
									"Woooo!! unable to display ad, We got unsupported ad type for interstitial. AdType : "
											+ adResponse.getAdType());

					// supported size ad isn't supplied so return null
					// onAdViewFailure();
					return;
				}
				if (activityIndicator != null) {

					addView(activityIndicator);
				}

			} else {
				MdotMLogger.e(this,
						"Unable to create bitmap image. is cached locally  "
								+ adResponse.isCachedLocally());
				adLoadSuccess = false;
				return;
			}
			drawBackgroundView(true);

			final int visibility = InterstitialImageView.super.getVisibility();
			setVisibility(visibility);

		} else {
			adLoadSuccess = false;
			return;
		}

	}

	private void drawBackgroundView(boolean isDefaultBackground) {
		MdotMLogger.d(this, " drawBackgroundView   :: " + isDefaultBackground);

		if (adSelectedBackground == null) {
			adSelectedBackground = populateDrawablwBackGround(
					HIGHLIGHT_BACKGROUND_COLOR, HIGHLIGHT_COLOR);
		}

		if (defaultBackground == null) {
			defaultBackground = populateDrawablwBackGround(Color.BLACK,
					backgroundColor);
		}
		if (isDefaultBackground) {
			setBackgroundDrawable(defaultBackground);
		} else {
			setBackgroundDrawable(adSelectedBackground);
		}
	}

	private static Drawable populateDrawablwBackGround(int backgroundColor,
			int color) {
		MdotMLogger.i("", "populate backgnd called");
		Rect rect = new Rect(0, 0, 1, 1);
		MdotMLogger.i("",
				"rect width and height " + rect.width() + " " + rect.height());
		Bitmap bitmap = Bitmap.createBitmap(rect.width(), rect.height(),
				Bitmap.Config.ARGB_8888);
		Canvas canvas = new Canvas(bitmap);

		Paint paint = new Paint();
		paint.setColor(backgroundColor);
		paint.setAntiAlias(true);
		canvas.drawRect(rect, paint);

		return new BitmapDrawable(bitmap);
	}

	private void createAdWithBannerView(Bitmap icon, MdotMAdResponse adResponse) {

		ImageView bannerView = new ImageView(getContext());
		bannerView.setImageBitmap(icon);
		bannerView.setScaleType(ScaleType.FIT_XY);

		// LayoutParams params = new LayoutParams(adResponse.getWidth(),
		// adResponse.getHeight());
		LayoutParams params = new LayoutParams(
				RelativeLayout.LayoutParams.WRAP_CONTENT,
				RelativeLayout.LayoutParams.WRAP_CONTENT);
		params.addRule(RelativeLayout.CENTER_IN_PARENT);

		bannerView.setLayoutParams(params);
		bannerView.setId(1);
		addView(bannerView);
		adLoadSuccess = true;
	}

	/**
	 * fetches the image using the supplied url
	 * 
	 * @param imageURL
	 * @param useCaches
	 * @return
	 */
	private Bitmap fetchImage(String imageURL, boolean useCaches) {
		Bitmap image = null;
		if (imageURL != null) {
			InputStream is = null;
			try {
				// Open a new URL and get the InputStream to load data from it.
				HttpParams httpParameters = new BasicHttpParams();
				// Set the timeout in milliseconds until a connection is
				// established.
				// The default value is zero, that means the timeout is not
				// used.
				int timeoutConnection = 5000;
				HttpConnectionParams.setConnectionTimeout(httpParameters,
						timeoutConnection);
				// Set the default socket timeout (SO_TIMEOUT)
				// in milliseconds which is the timeout for waiting for data.
				int timeoutSocket = 5000;
				HttpConnectionParams
						.setSoTimeout(httpParameters, timeoutSocket);

				HttpClient httpClient = new DefaultHttpClient(httpParameters);
				HttpResponse response = httpClient
						.execute(new HttpGet(imageURL));
				if (response.getStatusLine().getStatusCode() == 200) {
					is = response.getEntity().getContent();
					// Create a bitmap.
					image = BitmapFactory.decodeStream(is);
					// Note we do not want to save the image to the file system.
				}

				
			} catch (Exception e) {
				image = null;
				MdotMLogger.e(this, "Problem while fetchImage()  :  " + e);
			} finally {
				if (is != null) {
					try {
						is.close();
					} catch (IOException e) {
						// We are ignoring and continuing
					}
				}
			}
		} else {
			MdotMLogger.e(this, "Image url is null");
		}
		return image;
	}

	private void setAdSelectionInProgress(boolean isProgress) {
		this.adSelectionInProgress = isProgress;
	}

	private boolean isAdSelectionInProgress() {
		return adSelectionInProgress;
	}

	@Override
	public boolean dispatchKeyEvent(KeyEvent event) {
		int action = event.getAction();
		if (!this.isAdClicked) {
			if (action == MotionEvent.ACTION_UP) {
				isAdClicked = true;
				clicked(this.getContext());
			}
		}
		return super.dispatchKeyEvent(event);
	}

	// @Override
	// public boolean dispatchTouchEvent(MotionEvent motionEvent) {
	// int action = motionEvent.getAction();
	// float x = motionEvent.getX();
	// float y = motionEvent.getY();
	//
	// int left = getLeft();
	// int top = getTop();
	// int right = getRight();
	// int bottom = getBottom();
	//
	// MdotMLogger.i(this, "  Selected  ");
	// if (x < left || x > right || y < top || y > bottom) {
	// MdotMLogger.i(this, "Always outside of ad display area ");
	// if (!this.isAdClicked) {
	// drawBackgroundView(true);
	// }
	// } else {
	// if (action == MotionEvent.ACTION_UP) {
	// if (!this.isAdClicked) {
	// isAdClicked = true;
	// setClickable(false);
	// clicked(this.getContext());
	// drawBackgroundView(true);
	// }
	// } else if (action == MotionEvent.ACTION_DOWN) {
	// if (!this.isAdClicked) {
	// drawBackgroundView(false);
	// }
	// }
	// }
	// return super.dispatchTouchEvent(motionEvent);
	// }

	private void clicked(Context context) {
		MdotMLogger.i(this, "  Selected to clicked  ");

		if (this.adLandingUrl != null) {
			if (!isAdSelectionInProgress()) {
				final String clickedUrl = this.adLandingUrl;
				setAdSelectionInProgress(true);
				showActivityIndicator();
				new Thread() {

					public void run() {

						mAdClickListener.adClicked();
						MdotMLogger.i(this, "Launch type is " + launchType);

						if (launchType == 2) {
							Intent i = new Intent(Intent.ACTION_VIEW,
									Uri.parse(clickedUrl));
							i.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

							try {

								getContext().startActivity(i);

							} catch (Exception e) {

								MdotMLogger.i(this,
										"Could not open browser on ad click to "
												+ e);
							}

							mAdClickListener.leaveApplication();
							adNetworkCompleted();

						} else {

							handler.post(new Runnable() {

								public void run() {
									webView.loadUrl(clickedUrl);
								}
							});
						}

					}
				}.start();
			} else {
				MdotMLogger.i(this, "ad selection under progress");
			}
		} else {
			MdotMLogger.i(this, "selected ad is null");
		}
	}

	private void adNetworkCompleted() {
		MdotMLogger.i(this, "On ad network completed");
		setClickable(true);
		setAdSelectionInProgress(false);
		isAdClicked = false;
		hideActivityIndicator();
	}

	private void showActivityIndicator() {
		post(new Thread() {
			public void run() {
				if (activityIndicator != null) {
					activityIndicator.setVisibility(VISIBLE);
					// activityIndicator.setBackgroundDrawable(defaultBackground);
				}
			}
		});
	}

	private void hideActivityIndicator() {
		post(new Thread() {
			public void run() {
				if (activityIndicator != null) {
					activityIndicator.setVisibility(INVISIBLE);
					// setBackgroundDrawable(defaultBackground);
				}
			}
		});
	}
}
