package com.mdotm.android.view;

import java.io.IOException;
import java.io.InputStream;
import java.lang.ref.WeakReference;
import java.net.URL;
import java.net.URLConnection;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.Typeface;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Handler;
import android.os.StrictMode;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.ViewGroup;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.ImageView;
import android.widget.ImageView.ScaleType;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.mdotm.android.listener.MdotMAdActionListener;
import com.mdotm.android.model.MdotMAdResponse;
import com.mdotm.android.utils.MdotMLogger;
import com.mdotm.android.utils.MdotMUtils;

/**
 * @author Bheema
 * 
 */
class MdotMBannerView extends RelativeLayout {

	private boolean adSelectionInProgress;
	private String adLandingUrl;
	public boolean adLoadSuccess;
	private MdotMAdActionListener mAdClickListener;
	private ProgressBar activityIndicator;
	public static MdotMBannerView mBannerView = null;
	// Bitmap icon = null;
	private WeakReference<Bitmap> bannerBitmap;

	private int padding;
	private int textColor = Color.WHITE;
	private int backgroundColor = Color.BLACK;

	private static Drawable defaultBackground;

	private boolean isAdClicked = false;
	private WebView webView = null;
	private int launchType;
	Handler handler;
	private static final Typeface AD_FONT = Typeface.create(
			Typeface.SANS_SERIF, Typeface.BOLD);

	/**
	 * Constructs an advertisement view from a layout XML file.
	 * 
	 * @param context
	 *            is the application's context.
	 * @param attrs
	 *            are attributes set in the XML layout for this view.
	 * @param defStyle
	 *            is the theme ID to apply to this view.
	 * 
	 * @see android.view.View#View(Context, AttributeSet, int)
	 */
	public MdotMBannerView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
	}

	/**
	 * Constructs an advertisement view from a layout XML file.
	 * 
	 * @param context
	 *            is the application's context.
	 * @param attrs
	 *            are attributes set in the XML layout for this view.
	 * 
	 * @see android.view.View#View(android.content.Context,
	 *      android.util.AttributeSet)
	 */
	public MdotMBannerView(Context context, AttributeSet attrs) {
		this(context, attrs, 0);
	}

	public MdotMBannerView(Context context) {
		super(context);

	}

	public MdotMBannerView getInstance(Context context) {
		if (mBannerView == null) {
			mBannerView = new MdotMBannerView(context);
		}

		return mBannerView;
	}

	public void drawBannerImage(Context context, MdotMAdResponse adResponse,
			MdotMAdActionListener listener) {
		handler = new Handler();
		RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(
				adResponse.getWidth(), adResponse.getHeight());
		params.addRule(RelativeLayout.CENTER_IN_PARENT);

		this.setLayoutParams(params);
		mAdClickListener = listener;
		setFocusable(true);
		setDescendantFocusability(ViewGroup.FOCUS_AFTER_DESCENDANTS);
		setClickable(true);
		setAdSelectionInProgress(false);
		try {
			initializeAdView(adResponse, context);
		} catch (Exception e) {
			MdotMLogger.e(this, "Exception initializing banner adview");
			adLoadSuccess = false;
			return;
		}

	}

	@Override
	protected void onAttachedToWindow() {
		MdotMLogger.i(this, "on attached to window");
		super.onAttachedToWindow();
	}

	@Override
	protected void onDetachedFromWindow() {
		MdotMLogger.i(this, "on detached from window");
		super.onDetachedFromWindow();
	}

	@SuppressLint("NewApi")
	private void initializeAdView(MdotMAdResponse adResponse, Context context)
			throws Exception {
		if (adResponse != null) {
			launchType = adResponse.getLaunchType();
			webView = new WebView(context);
			webView.setFocusable(true);
			webView.getSettings().setJavaScriptEnabled(true);
			webView.setWebViewClient(new WebViewClient() {
				boolean timeout;
				Runnable run = new Runnable() {
					public void run() {

						if (timeout) {
							MdotMLogger.e(this, "timeout loading landing url");
							webView.stopLoading();
							adNetworkCompleted();
							timeout = false;
						}
					}
				};

				@Override
				public void onPageStarted(WebView view, String url,
						Bitmap favicon) {
					MdotMLogger.i(this, "started Loading url " + url);
					if (isAdSelectionInProgress()) {
						timeout = true;
						handler.removeCallbacks(run);
						handler.postDelayed(run, 15000);
					}
					super.onPageStarted(view, url, favicon);
				}

				@Override
				public boolean shouldOverrideUrlLoading(WebView view, String url) {
					MdotMLogger.i(this, "returned url " + url);
					if (url != null && url.startsWith("market://")) {
						view.getContext().startActivity(
								new Intent(Intent.ACTION_VIEW, Uri.parse(url)));
						mAdClickListener.leaveApplication();

						// Lets reset clickable and progress flag
						adNetworkCompleted();
						timeout = false;
						return true;
					} else {
						MdotMLogger.i(this, "loading other url " + url);
						return super.shouldOverrideUrlLoading(view, url);
					}
				}

				@Override
				public void onPageFinished(WebView view, String url) {
					super.onPageFinished(view, url);
					timeout = false;
					MdotMLogger.i(this, "Click page finished " + url);
					adNetworkCompleted();
				}

				@Override
				public void onReceivedError(WebView view, int errorCode,
						String description, String failingUrl) {
					super.onReceivedError(view, errorCode, description,
							failingUrl);
					timeout = false;
					if (isAdSelectionInProgress()) {
						MdotMLogger.e(this, "Unable to load landing url : "
								+ description);
					}
					// Lets reset clickable and progress flag
					adNetworkCompleted();
				}
			});
			this.adLandingUrl = adResponse.getLandingUrl();
			// The user can interact with the ad by clicking on it.
			setFocusable(true);
			setClickable(true);
			bannerBitmap = null;
			if (adResponse.isCachedLocally()) {
				bannerBitmap = new WeakReference<Bitmap>(
						BitmapFactory.decodeFile(adResponse.getImageUrl()));
				// icon = BitmapFactory.decodeFile(adResponse.getImageUrl());
			} else {
				if (android.os.Build.VERSION.SDK_INT > 9) {
					StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder()
							.permitNetwork().build();
					StrictMode.setThreadPolicy(policy);
				}
				bannerBitmap = new WeakReference<Bitmap>(fetchImage(
						adResponse.getImageUrl(), false));
				// icon = fetchImage(adResponse.getImageUrl(), false);

				if (android.os.Build.VERSION.SDK_INT > 9) {
					StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder()
							.detectNetwork().build();
					StrictMode.setThreadPolicy(policy);
				}
			}

			MdotMLogger.i(this, "Image is " + adResponse.getImageUrl());
			padding = 8;
			if (bannerBitmap.get() != null) {
				MdotMLogger.d(this, "The icon is nou null");
				activityIndicator = new ProgressBar(getContext());
				activityIndicator.setIndeterminate(false);
				LayoutParams activityParams = new LayoutParams(40, 40);
				activityIndicator.setLayoutParams(activityParams);
				activityIndicator.setVisibility(INVISIBLE);
				activityIndicator.setMinimumHeight(8);
				activityIndicator.setMinimumWidth(8);
				activityIndicator.setMax(100);
				activityIndicator.setBackgroundDrawable(defaultBackground);

				// set the attributes.
				if (adResponse.getAdType() == MdotMUtils.AD_WITH_BANNER_OR_XHTML_OR_VIDEO) {
					createAdWithBannerView(bannerBitmap.get(), adResponse);
					if (activityIndicator != null) {
						activityIndicator.setId(2); // for alignment.
						activityParams.addRule(CENTER_IN_PARENT);
					}

				} else if (adResponse.getAdType() == MdotMUtils.AD_ICON_WITH_TEXT_MESSAGE) {

					createAdWithIconView(bannerBitmap.get(), adResponse);
					if (activityIndicator != null) {
						activityIndicator.setId(3); // for alignment.
						activityParams.addRule(CENTER_IN_PARENT);
					}
				} else {
					adLoadSuccess = false;
					MdotMLogger.e(this,
							"Woooo!! unable to display ad, We got unsupported ad type. : "
									+ adResponse.getAdType());

					return;
				}
				if (activityIndicator != null) {

					addView(activityIndicator);
				}

			} else {
				MdotMLogger.d(this, "The icon is null");
				adLoadSuccess = false;
				return;
			}

			final int visibility = MdotMBannerView.super.getVisibility();
			setVisibility(visibility);

		} else {
			adLoadSuccess = false;
			return;
		}

	}

	private void createAdWithBannerView(Bitmap icon, MdotMAdResponse adResponse) {

		ImageView bannerView = new ImageView(getContext());
		bannerView.setImageBitmap(icon);
		bannerView.setScaleType(ScaleType.FIT_XY);
		MdotMLogger.d(this, "Creating ad with bannerView view");
		LayoutParams params = new LayoutParams(adResponse.getWidth(),
				adResponse.getHeight());
		params.addRule(RelativeLayout.CENTER_IN_PARENT);

		bannerView.setLayoutParams(params);
		bannerView.setId(1);
		MdotMLogger.d(this, "The banner view is:" + (bannerView == null));
		addView(bannerView);
		adLoadSuccess = true;
	}

	private void createAdWithIconView(Bitmap icon, MdotMAdResponse ad) {
		this.setBackgroundColor(backgroundColor);
		ImageView iconView = new ImageView(getContext());
		iconView.setImageBitmap(icon);
		LayoutParams params = new LayoutParams(icon.getWidth(),
				icon.getHeight());
		params.setMargins(padding, padding, 0, padding);
		iconView.setLayoutParams(params);

		iconView.setId(1);
		addView(iconView);

		TextView adTextView = new TextView(getContext());
		adTextView.setText(ad.getText());
		adTextView.setContentDescription(ad.getText());
		adTextView.setTypeface(AD_FONT);
		adTextView.setTextColor(textColor);
		adTextView.setTextSize(13.0f);
		adTextView.setId(2);

		LayoutParams adTextViewParams = new LayoutParams(
				ViewGroup.LayoutParams.WRAP_CONTENT,
				ViewGroup.LayoutParams.WRAP_CONTENT);
		// if the icon was not null then align with that too.
		if (icon != null) {

			adTextViewParams.addRule(RelativeLayout.RIGHT_OF, 2);
		}

		// left top right bottom
		adTextViewParams.setMargins(50, 4, 10, 4);
		adTextViewParams.addRule(CENTER_IN_PARENT);
		// adTextViewParams.addRule(ALIGN_PARENT_TOP);
		adTextView.setLayoutParams(adTextViewParams);

		addView(adTextView);
		setGravity(CENTER_HORIZONTAL | CENTER_VERTICAL);
		adLoadSuccess = true;
	}

	/**
	 * fetches the image using the supplied url
	 * 
	 * @param imageURL
	 * @param useCaches
	 * @return
	 */
	private Bitmap fetchImage(String imageURL, boolean useCaches) {
		Bitmap image = null;
		if (imageURL != null) {
			InputStream is = null;
			try {
				// Open a new URL and get the InputStream to load data from it.
				HttpParams httpParameters = new BasicHttpParams();
				// Set the timeout in milliseconds until a connection is
				// established.
				// The default value is zero, that means the timeout is not
				// used.
				int timeoutConnection = 5000;
				HttpConnectionParams.setConnectionTimeout(httpParameters,
						timeoutConnection);
				// Set the default socket timeout (SO_TIMEOUT)
				// in milliseconds which is the timeout for waiting for data.
				int timeoutSocket = 5000;
				HttpConnectionParams
						.setSoTimeout(httpParameters, timeoutSocket);

				HttpClient httpClient = new DefaultHttpClient(httpParameters);
				HttpResponse response = httpClient
						.execute(new HttpGet(imageURL));
				if (response.getStatusLine().getStatusCode() == 200) {
					is = response.getEntity().getContent();
					// Create a bitmap.
					image = BitmapFactory.decodeStream(is);
					// Note we do not want to save the image to the file system.
				}

			} catch (Exception e) {
				image = null;
				MdotMLogger.e(this, "Problem while fetchImage()  :  " + e);
			} finally {
				if (is != null) {
					try {
						is.close();
					} catch (IOException e) {
						// We are ignoring and continuing
					}
				}
			}
		} else {
			MdotMLogger.e(this, "Image url is null");
		}
		return image;
	}

	private void setAdSelectionInProgress(boolean isProgress) {
		this.adSelectionInProgress = isProgress;
	}

	private boolean isAdSelectionInProgress() {
		return adSelectionInProgress;
	}

	@Override
	public boolean dispatchKeyEvent(KeyEvent event) {
		int action = event.getAction();
		if (!this.isAdClicked) {
			if (action == MotionEvent.ACTION_UP) {
				isAdClicked = true;
				clicked();
			}
		}
		return super.dispatchKeyEvent(event);
	}

	@Override
	public boolean dispatchTouchEvent(MotionEvent motionEvent) {
		int action = motionEvent.getAction();
		float x = motionEvent.getX();
		float y = motionEvent.getY();

		int left = getLeft();
		int top = getTop();
		int right = getRight();
		int bottom = getBottom();

		MdotMLogger.i(this, "  Selected  ");
		if (x < left || x > right || y < top || y > bottom) {
			MdotMLogger.i(this, "Always outside of ad display area ");
			if (!this.isAdClicked) {
			}
		} else {
			if (action == MotionEvent.ACTION_UP) {
				if (!this.isAdClicked) {
					isAdClicked = true;
					setClickable(false);
					clicked();
				}
			} else if (action == MotionEvent.ACTION_DOWN) {
				if (!this.isAdClicked) {

				}
			}
		}
		return super.dispatchTouchEvent(motionEvent);
	}

	private void clicked() {
		MdotMLogger.i(this, "  Selected to clicked  is selection in progress "
				+ isAdSelectionInProgress());

		if (this.adLandingUrl != null) {
			if (!isAdSelectionInProgress()) {
				setAdSelectionInProgress(true);
				final String clickedUrl = this.adLandingUrl;

				showActivityIndicator();
				new Thread() {

					public void run() {

						MdotMLogger.i(this, "Destination url is " + clickedUrl);
						mAdClickListener.adClicked();
						MdotMLogger.i(this, "Launch type is " + launchType);
						if (launchType == 2) {
							Intent i = new Intent(Intent.ACTION_VIEW,
									Uri.parse(clickedUrl));
							i.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

							try {

								getContext().startActivity(i);

							} catch (Exception e) {

								MdotMLogger.i(this,
										"Could not open browser on ad click to "
												+ e);
							}

							mAdClickListener.leaveApplication();
							adNetworkCompleted();

						} else {

							handler.post(new Runnable() {

								public void run() {
									webView.loadUrl(clickedUrl);
								}
							});

						}

					}
				}.start();
			} else {
				MdotMLogger.i(this, "ad selection under progress");
			}
		} else {
			MdotMLogger.i(this, "selected ad is null");
		}
	}

	private void adNetworkCompleted() {
		MdotMLogger.i(this, "On ad network completed");
		setClickable(true);
		setAdSelectionInProgress(false);
		isAdClicked = false;
		hideActivityIndicator();
	}

	private void showActivityIndicator() {
		post(new Thread() {
			public void run() {
				if (activityIndicator != null) {
					activityIndicator.setVisibility(VISIBLE);
					activityIndicator.setBackgroundDrawable(defaultBackground);
				}
			}
		});
	}

	private void hideActivityIndicator() {
		post(new Thread() {
			public void run() {
				if (activityIndicator != null) {
					activityIndicator.setVisibility(INVISIBLE);
					setBackgroundDrawable(defaultBackground);
				}
			}
		});
	}
}
