package com.mdotm.android.view;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.widget.RelativeLayout;

import com.mdotm.android.http.MdotMRequestQueueClient;
import com.mdotm.android.listener.MdotMAdEventListener;
import com.mdotm.android.listener.MdotMNetworkListener;
import com.mdotm.android.model.MdotMAdRequest;
import com.mdotm.android.model.MdotMAdResponse;
import com.mdotm.android.utils.MdotMLogger;
import com.mdotm.android.utils.MdotMUtils;

/**
 * @author Bheema
 * 
 */
public class MdotMInterstitial implements MdotMNetworkListener {
	public static MdotMAdEventListener mAdEventListener;
	public static boolean isInterstitialReady = false;
	static RelativeLayout mInterstitialView;
	Context mContext;
	int adType;
	private Handler handler;
	public static MdotMAdResponse interstitialResponse = null;

	public MdotMInterstitial(Context context) {
		handler = new Handler();
		mContext = context;
	}

	/**
	 * Call this method to load interstitial ad. User should call
	 * showInterstitial to display the ad
	 * 
	 * @param listener
	 *            - MdotMAdEventListener to communicate back to application
	 * @param adRequest
	 *            - request parameter object
	 * 
	 */
	public void loadInterstitial(MdotMAdEventListener listener,
			MdotMAdRequest adRequest) {
		mAdEventListener = listener;
		isInterstitialReady = false;
		adRequest.setLoadInterstitial(true);
		adRequest.setAdSize(MdotMUtils.getUtilsInstance().getDisplaySize(
				mContext));
		interstitialResponse = null;

		MdotMRequestQueueClient.getInstance(mContext).addRequest(adRequest,
				this);

	}

	/**
	 * Call this method to display interstitial ad. Developer should define
	 * com.mdotm.android.view.MdotMActivity in their manifest file.
	 * 
	 * @param activity
	 *            Pass activity context as a parameter.
	 */
	public void showInterstitial(final Activity activity) {

		if (isInterstitialReady) {
			handler.post(new Runnable() {

				public void run() {
					if (mAdEventListener != null) {
						mAdEventListener.willShowInterstitial();
					}
					int orientation = activity.getResources()
							.getConfiguration().orientation;

					Intent intent = new Intent(activity, MdotMActivity.class);
					intent.putExtra("response", interstitialResponse);
					intent.putExtra("ScreenOrientation", orientation);
					activity.startActivity(intent);
					if (mAdEventListener != null) {
						// mAdEventListener.onInterstitialPresentScreen();
						// mAdEventListener.onLeaveApplicationFromInterstitial();
						mAdEventListener.didShowInterstitial();
					}
				}
			});
		} else {
			MdotMLogger.i(this, "Interstitial is not ready");
		}
	}

	public void finishedAdParsing(MdotMAdResponse adResponse) {
		if (adResponse != null && adResponse.getStatus() != 0
				&& adResponse.getImageUrl() != null) {
			interstitialResponse = adResponse;
			isInterstitialReady = true;
			if (mAdEventListener != null) {
				mAdEventListener.onReceiveInterstitialAd();
			}
		} else {
			if (mAdEventListener != null) {
				mAdEventListener.onFailedToReceiveInterstitialAd();
			}
		}

	}

}
