package com.mdotm.android.view;

import android.app.ProgressDialog;
import android.content.Context;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.media.MediaPlayer.OnErrorListener;
import android.media.MediaPlayer.OnPreparedListener;
import android.os.Handler;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.VideoView;

import com.mdotm.android.listener.MdotMAdActionListener;
import com.mdotm.android.model.MdotMAdResponse;
import com.mdotm.android.utils.MdotMLogger;
import com.mdotm.android.utils.MdotMUtils;

/**
 * @author Bheema
 * 
 */
class MdotMVideoView extends VideoView implements OnPreparedListener,
		OnCompletionListener, OnErrorListener {

	MdotMAdResponse mAdResponse;
	MdotMAdActionListener adActionListener;
	Context mContext;
	ProgressDialog mProgressDialog;
	private boolean mDidImpressionRecoded = false;
	boolean isVideoReady = false;
	boolean mIsGetIntoError = false;

	// WebView impressionReportView;

	public MdotMVideoView(Context context, final MdotMAdResponse adResponse,
			MdotMAdActionListener listener) {
		super(context);
		mContext = context;
		mAdResponse = adResponse;
		adActionListener = listener;
		isVideoReady = false;
		mIsGetIntoError = false;
		mDidImpressionRecoded = false;
		mProgressDialog = new ProgressDialog(context);
		mProgressDialog.setMessage("Loading...");
		mProgressDialog.setCanceledOnTouchOutside(false);
		mProgressDialog.show();
		this.requestFocus();
		this.setOnPreparedListener(this);
		this.setOnCompletionListener(this);
		this.setOnErrorListener(this);
		this.setVideoPath(adResponse.getImageUrl());

		setFocusable(true);

	}

	public void onPrepared(MediaPlayer mp) {
		MdotMLogger.i(this, "onPrepared");
		isVideoReady = true;
		mProgressDialog.dismiss();
		mp.start();

	}

	public void PauseVideo() {
		if (this.isPlaying()) {
			this.pause();
		}
	}

	public void stopVideo(Handler handler) {
		MdotMLogger.i(this, "Stop video playback");
		if (this.isPlaying()) {
			this.stopPlayback();
		}
		if (!mIsGetIntoError && !mDidImpressionRecoded) {
			MdotMLogger.i(this, "Call report impression video resource");
			handler.post(new Runnable() {

				public void run() {
					// impressionReportView = new WebView(mContext);
					MdotMLogger.i(this, "Call report impression image");

					// impressionReportView.setWebViewClient(new WebViewClient()
					// {
					// @Override
					// public void onPageFinished(WebView view, String url) {
					// MdotMLogger.i(this, "Reported the impression "
					// + url);
					// super.onPageFinished(view, url);
					// mDidImpressionRecoded = true;
					// }
					// });
					// impressionReportView.getSettings().setJavaScriptEnabled(
					// true);
					// impressionReportView.loadUrl(mAdResponse.getImp_pixel());
					MdotMUtils.getUtilsInstance().reportImpression(
							mAdResponse.getImp_pixel(), mContext);
					mDidImpressionRecoded = true;
				}
			});

		}
	}

	public void playVideo() {
		if (!this.isPlaying()) {
			this.start();
		}
	}

	public void onCompletion(MediaPlayer mp) {
		MdotMLogger.i(this, "On completion of video");
		adActionListener.onPlayStop();
	}

	public boolean onError(MediaPlayer mp, int what, int extra) {
		MdotMLogger.i(this, "On error playing  video " + what + " extra "
				+ extra);

		mIsGetIntoError = true;
		if (mProgressDialog != null && mProgressDialog.isShowing()) {
			mProgressDialog.dismiss();
		}
		adActionListener.onPlayError();
		return true;
	}

	public boolean isReadyToPlay() {
		return isVideoReady;
	}

}
