package com.mdotm.android.view;

import java.util.Timer;
import java.util.TimerTask;

import android.content.Context;
import android.content.Intent;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Typeface;
import android.net.Uri;
import android.os.Handler;
import android.util.AttributeSet;
import android.view.View;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.VideoView;

import com.mdotm.android.http.MdotMNetworkManager;
import com.mdotm.android.http.MdotMRequestQueueClient;
import com.mdotm.android.listener.MdotMAdActionListener;
import com.mdotm.android.listener.MdotMAdEventListener;
import com.mdotm.android.listener.MdotMNetworkListener;
import com.mdotm.android.model.MdotMAdRequest;
import com.mdotm.android.model.MdotMAdResponse;
import com.mdotm.android.utils.MdotMLogger;
import com.mdotm.android.utils.MdotMUtils;

/**
 * @author Bheema
 * 
 */
class MdotMView extends RelativeLayout implements MdotMNetworkListener,
		MdotMAdActionListener {
	private MdotMAdEventListener mAdEventListener = null;
	private MdotMAdRequest mAdRequest = null;
	private Context mContext;
	Handler mHandler;
	MdotMNetworkManager mAdManager;
	Timer timer;
	private boolean mDidApllicationLeave = false;
	private boolean mAdViewHasFocus = true;
	public boolean didParentStopped = false;

	// WebView impressionReportView;

	public MdotMView(Context context) {
		super(context);
		this.mContext = context;
		mHandler = new Handler();
		RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(
				RelativeLayout.LayoutParams.WRAP_CONTENT,
				RelativeLayout.LayoutParams.WRAP_CONTENT);
		params.addRule(RelativeLayout.CENTER_IN_PARENT);
		this.setLayoutParams(params);
	}

	public MdotMView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
	}

	public MdotMView(Context context, AttributeSet attrs) {
		super(context, attrs);
		this.mContext = context;
		mHandler = new Handler();
		RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(
				RelativeLayout.LayoutParams.WRAP_CONTENT,
				RelativeLayout.LayoutParams.WRAP_CONTENT);
		params.addRule(RelativeLayout.CENTER_IN_PARENT);
		this.setLayoutParams(params);
	}

	@Override
	public void onWindowFocusChanged(boolean hasWindowFocus) {
		super.onWindowFocusChanged(hasWindowFocus);
		MdotMLogger.i(this, "Window focused " + hasWindowFocus);
		if (hasWindowFocus) {
			mAdViewHasFocus = true;
			MdotMRequestQueueClient.getInstance(mContext)
					.setApplicationHasFocus(mAdViewHasFocus);
		} else {
			mAdViewHasFocus = false;
			MdotMRequestQueueClient.getInstance(mContext)
					.setApplicationHasFocus(mAdViewHasFocus);
		}
		if (hasWindowFocus && mDidApllicationLeave) {
			MdotMLogger.i(this, "Focus gained " + hasWindowFocus);
			mDidApllicationLeave = false;
			onDismissScreen();
		}
	}

	@Override
	protected void onDraw(Canvas canvas) {
		// if (mImpressionReported) {
		// if (impressionReportView != null) {
		// impressionReportView.destroy();
		// impressionReportView = null;
		// }
		// }
		super.onDraw(canvas);
	}

	public void loadAd(MdotMAdEventListener listener, MdotMAdRequest adRequest) {
		mAdEventListener = listener;
		mAdRequest = adRequest;
		sendRequest();

	}

	private void sendRequest() {
		MdotMLogger.i(this, "Sending request ");
		String[] sizes = mAdRequest.getAdSize().split(",");
		int width = getPixels(Integer.parseInt(sizes[0]));
		int height = getPixels(Integer.parseInt(sizes[1]));

		mAdRequest.setAdSize(width + "," + height);

		// MdotMLogger.i(this, "original w = " + sizes[0] + " h = " + sizes[1]
		// + " converted pixel w " + width + " h " + height);
		if (mAdRequest.getAdRefreshInterval() > 0) {
			scheduleTimer();
		} else if (mAdViewHasFocus) {
			MdotMLogger.i(this, "MdotMView is visible so requesting for ad");
			MdotMRequestQueueClient.getInstance(mContext).addRequest(
					mAdRequest, this);
			MdotMRequestQueueClient.getInstance(mContext)
					.setApplicationHasFocus(mAdViewHasFocus);
		}

	}

	private int getPixels(int dip) {
		final float scale = mContext.getResources().getDisplayMetrics().density;
		int value = (int) (dip * scale + 0.5f);
		return value;
	}

	private void scheduleTimer() {
		int delay = 1000; // delay for 5 sec.

		int period = mAdRequest.getAdRefreshInterval() * 1000; // repeat every
		// adRequest.getAdRefreshInterval()
		// sec.

		timer = new Timer();

		timer.scheduleAtFixedRate(new TimerTask() {

			public void run() {

				mHandler.post(new Runnable() {

					public void run() {
						if (mAdViewHasFocus) {

							MdotMLogger.i(this, "MdotMView has focus"
									+ mAdViewHasFocus);
							MdotMLogger.i(this, "Begining next request");
							MdotMRequestQueueClient.getInstance(mContext)
									.addRequest(mAdRequest, MdotMView.this);

						} else {
							MdotMLogger
									.i(this,
											"MdotMView does not has focus. So Ad request cancelled");
						}

					}
				});

			}

		}, delay, period);

	}

	public void finishedAdParsing(final MdotMAdResponse adResponse) {
		if (adResponse != null && adResponse.getStatus() != 0) {
			if (adResponse.isRequestForBannerAd()) {

				mHandler.post(new Runnable() {

					public void run() {
						MdotMView.this.removeAllViews();

						View view = createBannerView(adResponse);
						if (view != null) {
							addView(view);
							if (mAdEventListener != null) {
								mAdEventListener.onReceiveBannerAd();
							}
						} else {
							if (mAdEventListener != null) {
								mAdEventListener.onFailedToReceiveBannerAd();
							}
						}
					}
				});
			}
		} else {
			if (mAdEventListener != null) {
				mAdEventListener.onFailedToReceiveBannerAd();
			}
		}

	}

	public View createBannerView(MdotMAdResponse adResponse) {
		// MdotMLogger.i(this, "AdResponse is null " + (adResponse == null));
		// MdotMLogger.i(this, "ad type is " + adResponse.getAdType());
		// impressionReportView = new WebView(this.getContext());
		View view = null;
		if (!didParentStopped) {
			if (adResponse.getAdType() == MdotMUtils.AD_WITH_BANNER_OR_XHTML_OR_VIDEO
					|| adResponse.getAdType() == MdotMUtils.AD_ICON_WITH_TEXT_MESSAGE) {
				if (adResponse.getResourceType() == MdotMNetworkManager.imageResource
						&& adResponse.getAdType() == MdotMUtils.AD_WITH_BANNER_OR_XHTML_OR_VIDEO) {
					view = new MdotMBannerView(mContext);
					((MdotMBannerView) view).drawBannerImage(mContext,
							adResponse, this);
					MdotMLogger.i(this, "continuing");
					if (!((MdotMBannerView) view).adLoadSuccess) {
						view = null;
					} else if (mAdViewHasFocus && !didParentStopped) {
						MdotMLogger.i(this, "Call report impression image");
						MdotMUtils.getUtilsInstance().reportImpression(
								adResponse.getImp_pixel(), this.getContext());

						// impressionReportView.setWebViewClient(new
						// WebViewClient()
						// {
						// @Override
						// public void onPageFinished(WebView view, String url)
						// {
						// MdotMLogger.i(this, "Reported the impression "
						// + url);
						// super.onPageFinished(view, url);
						// mImpressionReported = true;
						// }
						// });
						// impressionReportView.getSettings().setJavaScriptEnabled(
						// true);
						// impressionReportView.loadUrl(adResponse.getImp_pixel());
					}
				} else if (adResponse.getResourceType() == MdotMNetworkManager.htmlResource
						|| adResponse.getResourceType() == MdotMNetworkManager.gifImageResource
						|| (adResponse.getResourceType() == MdotMNetworkManager.imageResource && adResponse
								.getAdType() == MdotMUtils.AD_ICON_WITH_TEXT_MESSAGE)) {

					if (mAdViewHasFocus) {
						// loading web page if window has focus. If it has
						// then page will be loaded and impression will reported
						view = new HtmlView(mContext, adResponse, this);
					}
				} else if (adResponse.getResourceType() == MdotMNetworkManager.videoResource) {
					// TODO currently video ad is not supported as part of
					// banner
					// size ad
					MdotMLogger.e(this, "Video is not supported");
					view = null;
					// view = getVideoViewLayout(adResponse);
					// if (mAdViewHasFocus) {
					// MdotMLogger.i(this, "Call report impression video");
					// MdotMUtils.getUtilsInstance().reportImpression(
					// adResponse.getImp_pixel(), this.getContext());
					// }

				}
			}
		}
		return view;
	}

	private View getVideoViewLayout(final MdotMAdResponse response) {
		MdotMLogger.e(this, "The response is:" + response.getWidth() + "::"
				+ response.getHeight());
		RelativeLayout videoLayout = new RelativeLayout(mContext);
		RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(
				LayoutParams.FILL_PARENT, response.getHeight());
		videoLayout.setLayoutParams(params);
		params.addRule(RelativeLayout.CENTER_IN_PARENT);

		RelativeLayout closeLayout = new RelativeLayout(mContext);
		RelativeLayout.LayoutParams closeLayoutparams = new RelativeLayout.LayoutParams(
				RelativeLayout.LayoutParams.WRAP_CONTENT,
				RelativeLayout.LayoutParams.WRAP_CONTENT);
		closeLayoutparams.addRule(RelativeLayout.ALIGN_PARENT_RIGHT);
		closeLayoutparams.setMargins(0, 2, 10, 0);

		closeLayout.setLayoutParams(closeLayoutparams);
		TextView closeText = new TextView(mContext);
		closeText.setText("Learn More");
		closeText.setTextColor(Color.WHITE);
		closeText.setTypeface(null, Typeface.BOLD);
		closeLayout.addView(closeText);

		closeText.setOnClickListener(new OnClickListener() {

			public void onClick(View v) {
				MdotMView.this.adClicked();
				MdotMLogger.i(this, "video clicked");
				Intent browserIntent = new Intent("android.intent.action.VIEW",
						Uri.parse(response.getLandingUrl()));
				mContext.startActivity(browserIntent);
			}
		});

		VideoView view = new MdotMVideoView(mContext, response, this);
		RelativeLayout.LayoutParams videoParams = new RelativeLayout.LayoutParams(
				response.getWidth(), response.getHeight());
		videoParams.addRule(RelativeLayout.CENTER_IN_PARENT);
		view.setLayoutParams(videoParams);
		videoLayout.addView(view);
		videoLayout.addView(closeLayout);
		return videoLayout;
	}

	public void adClicked() {
		mAdEventListener.onBannerAdClick();
		// mAdEventListener.onBannerPresentScreen();

	}

	public void leaveApplication() {
		mDidApllicationLeave = true;
		mAdEventListener.onLeaveApplicationFromBanner();

	}

	public void onDismissScreen() {
		mAdEventListener.onBannerDismissScreen();

	}

	public void onPlayError() {

	}

	public void onPlayStop() {

	}

	public void onLoadError() {

	}

}
