package com.mdotm.android.view;

import android.annotation.TargetApi;
import android.app.Activity;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.os.Build;
import android.util.AttributeSet;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.webkit.GeolocationPermissions;
import android.webkit.WebChromeClient;
import android.webkit.WebSettings;
import android.webkit.WebSettings.PluginState;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;

import com.mdotm.android.http.MdotMNetworkManager;
import com.mdotm.android.listener.MdotMAdActionListener;
import com.mdotm.android.model.MdotMAdResponse;
import com.mdotm.android.utils.MdotMLogger;
import com.mdotm.android.utils.MdotMUtils;

/**
 * @author Meghana,Bheema
 * 
 */
@TargetApi(Build.VERSION_CODES.HONEYCOMB)
class MdotMWebView extends WebView {
	private Context mContext;
	private MyWebChromeClient mWebChromeClient;
	private View mCustomView;
	private FrameLayout mCustomViewContainer;
	private WebChromeClient.CustomViewCallback mCustomViewCallback;

	private FrameLayout mContentView;
	private FrameLayout mBrowserFrameLayout;
	private FrameLayout mLayout;

	private MdotMAdActionListener mActionListener;
	private MdotMAdResponse mAdResponse;

	private boolean mHtmlLoadError = false;
	private boolean isWebViewHasFocus = true;
	public boolean mIsImpReported = false;

	// WebView impressionReportView;

	public MdotMWebView(Context context, AttributeSet attrs) {
		super(context, attrs);
		init(context);
	}

	public MdotMWebView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		init(context);
	}

	public MdotMWebView(Context context, AttributeSet attrs, int defStyle,
			boolean privateBrowsing) {
		super(context, attrs, defStyle, privateBrowsing);
		init(context);
	}

	public MdotMWebView(Context context, MdotMAdResponse adResponse,
			MdotMAdActionListener listener) {
		super(context);
		mHtmlLoadError = false;
		mIsImpReported = false;
		mActionListener = listener;
		mAdResponse = adResponse;
		init(context);
		RelativeLayout.LayoutParams webParams = new RelativeLayout.LayoutParams(
				adResponse.getWidth(), adResponse.getHeight());
		webParams.addRule(RelativeLayout.CENTER_IN_PARENT);

		this.setLayoutParams(webParams);

		this.setBackgroundColor(Color.TRANSPARENT);
	}

	public void onWindowFocusChanged(boolean hasWindowFocus) {
		isWebViewHasFocus = hasWindowFocus;
		MdotMLogger.i(this, "Webview has focus " + isWebViewHasFocus);

	};

	@Override
	protected void onDraw(Canvas canvas) {
		// MdotMLogger.i(this, "webview ondraw destroying impression webview ");
		// if (mIsImpReported) {
		// if (impressionReportView != null) {
		// impressionReportView.destroy();
		// impressionReportView = null;
		// }
		// }
		super.onDraw(canvas);
	}

	private void init(Context context) {

		mContext = context;
		mLayout = new FrameLayout(context);

		mBrowserFrameLayout = new FrameLayout(mContext);
		mContentView = new FrameLayout(mContext);
		mCustomViewContainer = new FrameLayout(mContext);
		mBrowserFrameLayout.addView(mCustomViewContainer);

		LinearLayout linearLayout = new LinearLayout(mContext);
		mBrowserFrameLayout.addView(linearLayout);
		linearLayout.addView(mContentView);

		mLayout.addView(mBrowserFrameLayout, COVER_SCREEN_PARAMS);
		WebSettings webSettings = getSettings();
		webSettings.setJavaScriptEnabled(true);

		this.setInitialScale(100);
		this.setVerticalScrollBarEnabled(false);
		this.setHorizontalScrollBarEnabled(false);

		webSettings.setAllowFileAccess(true);
		if (Build.VERSION.SDK_INT > 7) {
			webSettings.setPluginState(PluginState.ON);
		} else {
			webSettings.setPluginsEnabled(true);
		}
		mWebChromeClient = new MyWebChromeClient();
		setWebChromeClient(mWebChromeClient);
		setWebViewClient(new MyWebViewClient());
		MdotMLogger.i(this, "loading url " + mAdResponse.getImageUrl());
		if (mAdResponse.getResourceType() == MdotMNetworkManager.gifImageResource) {
			if (mAdResponse.getImageUrl() != null
					&& mAdResponse.getImageUrl().length() > 0
					&& mAdResponse.isCachedLocally()) {
				int pos = mAdResponse.getImageUrl().lastIndexOf("/");
				String url = mAdResponse.getImageUrl().substring(pos);
				String path = mAdResponse.getImageUrl().substring(0, pos);
				url = url.replace("/", "");
				MdotMLogger.d(this, "The url is:" + url + "::" + path);
				this.loadDataWithBaseURL(
						"file://" + path + "/",
						"<html><head> <meta name = \"viewport\" content = \" initial-scale = 1.0,maximum-scale = 1.0, user-scalable = no\"> <style type=\"text/css\"> #background { width: %ipx; height: %ipx; position: fixed; left: 0px; top: 0px;  z-index: -1; background-color : transparent; background-position : top; } .stretch { width:100%%;  } </style></head> <body bgColor:transparent style='margin:0;'><div id=\"background\"><img src = '"
								+ url
								+ "' class=\"stretch\" alt=\"\" /></div></body></html>",
						"text/html", "utf-8", null);
			} else if (mAdResponse.getImageUrl() != null
					&& mAdResponse.getImageUrl().length() > 0) {
				String html = "<html><head> <meta name = \"viewport\" content = \" initial-scale = 1.0,maximum-scale = 1.0, user-scalable = no\"> <style type=\"text/css\"> #background { width: %ipx; height: %ipx; position: fixed; left: 0px; top: 0px;  z-index: -1; background-color : transparent; background-position : top; } .stretch { width:100%%;  } </style></head> <body bgColor:transparent style='margin:0;'><div id=\"background\"><img src = '"
						+ mAdResponse.getImageUrl()
						+ "' class=\"stretch\" alt=\"\" /></div></body></html>";
				MdotMLogger.i(this, "html " + html);
				// this.loadUrl(html);
				this.loadDataWithBaseURL(null, html, "text/html", "utf-8", null);
				// this.loadData(html, "text/html", "utf-8");
			}
		} else if (mAdResponse.getResourceType() == MdotMNetworkManager.imageResource
				&& mAdResponse.getAdType() == MdotMUtils.AD_ICON_WITH_TEXT_MESSAGE) {
			if (mAdResponse.getImageUrl() != null
					&& mAdResponse.getImageUrl().length() > 0) {
				String fileName = null;
				String path = null;
				if (mAdResponse.isCachedLocally()) {
					int pos = mAdResponse.getImageUrl().lastIndexOf("/");
					fileName = mAdResponse.getImageUrl().substring(pos);
					path = mAdResponse.getImageUrl().substring(0, pos);
					fileName = fileName.replace("/", "");
					MdotMLogger.d(this, "The url is:" + fileName + "::" + path);
				}
				boolean isCachedLocally = mAdResponse.isCachedLocally();

				boolean useIcon = false;
				int textWidth = mAdResponse.getWidth();
				int fontSize = 10;

				if (mAdResponse.getHeight() < 90) {
					fontSize = getInDp(10);
					useIcon = true;
				}

				if (mAdResponse.getHeight() >= 90
						&& mAdResponse.getHeight() <= 250) {
					fontSize = getInDp(16);
					useIcon = false;
				}

				if (mAdResponse.getHeight() > 250
						&& mAdResponse.getHeight() <= 480) {
					fontSize = getInDp(20);
					useIcon = false;
				}

				if (mAdResponse.getHeight() > 480) {
					fontSize = getInDp(32);
					useIcon = false;
				}

				String html = "<html> <head> <style></style> </head> <body style='margin:0;'> <div style='background:black; width:"
						+ mAdResponse.getWidth()
						+ "px; height:"
						+ mAdResponse.getHeight()
						+ "px; position:relative;'> <table width="
						+ mAdResponse.getWidth()
						+ "px height="
						+ mAdResponse.getHeight()
						+ "px border=0 cellspacing=5> <tr>";

				if (useIcon) {
					textWidth = textWidth - 38;
					html += "<td width='38'>";
					if (isCachedLocally) {
						html += "<img border='0' width='38' height='38' src='"
								+ fileName + "'/>";
					} else {
						html += "<img border='0' width='38' height='38' src='"
								+ mAdResponse.getImageUrl() + "'/>";
					}
					html += "</td>";
				}

				html += "<td width='"
						+ textWidth
						+ "' style='text-align:center; vertical-align:middle;'><a style='color: white; font-size: "
						+ fontSize
						+ "pt; font-family: Arial; font-weight: bold; text-decoration: none;' href='[URL]' target='_top'>";
				html += mAdResponse.getText();
				html += "</a></td>";
				html += "</tr></table>";
				html += "</div></body></html>";
				if (isCachedLocally) {
					this.loadDataWithBaseURL("file://" + path + "/", html,
							"text/html", "utf-8", null);
				} else {
					MdotMLogger.i(this, "html " + html);
					// this.loadUrl(html);
					// this.loadData(html, "text/html", "utf-8");
					this.loadDataWithBaseURL(null, html, "text/html", "utf-8",
							null);
				}
			}

		} else {

			if (mAdResponse.getImageUrl().startsWith("http://")) {
				MdotMLogger.e(this,
						"The image url is:" + mAdResponse.getImageUrl());
				this.loadUrl(mAdResponse.getImageUrl());
			} else {
				MdotMLogger.i(this, "Coming here" + mAdResponse.getImageUrl());
				this.loadUrl("file://" + mAdResponse.getImageUrl());
			}
		}

		this.setFocusable(true);
		this.requestFocus();
		setDescendantFocusability(ViewGroup.FOCUS_AFTER_DESCENDANTS);
		setClickable(true);
	}

	private int getInDp(int pt) {
		final float scale = mContext.getResources().getDisplayMetrics().density;
		int value = (int) (pt * scale + 0.5f);
		return value;
	}

	private class MyWebViewClient extends WebViewClient {

		@Override
		public void onReceivedError(WebView view, int errorCode,
				String description, String failingUrl) {
			super.onReceivedError(view, errorCode, description, failingUrl);
			mActionListener.onLoadError();
			MdotMLogger.e(this, "HTML Loading error");
			mHtmlLoadError = true;
		}

		@Override
		public void onPageFinished(WebView view, String url) {
			super.onPageFinished(view, url);
			MdotMLogger.i(this, "HTML Loading finished");
			if (!mHtmlLoadError && isWebViewHasFocus && !mIsImpReported) {
				mIsImpReported = true;
				MdotMLogger.i(this, "Call report impression gif/html");
				MdotMUtils.getUtilsInstance().reportImpression(
						mAdResponse.getImp_pixel(), mContext);
				// if (!mIsImpReported) {
				// impressionReportView = new WebView(mContext);
				// impressionReportView.setWebViewClient(new WebViewClient() {
				// @Override
				// public void onPageFinished(WebView view, String url) {
				// MdotMLogger.i(this, "Reported the impression "
				// + url);
				// super.onPageFinished(view, url);
				// mIsImpReported = true;
				// }
				// });
				// impressionReportView.getSettings().setJavaScriptEnabled(
				// true);
				// impressionReportView.loadUrl(mAdResponse.getImp_pixel());
				// }
			}
		}

	}

	public FrameLayout getLayout() {
		return mLayout;
	}

	public boolean inCustomView() {
		return (mCustomView != null);
	}

	public void hideCustomView() {
		mWebChromeClient.onHideCustomView();
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			if ((mCustomView == null) && canGoBack()) {
				goBack();
				return true;
			}
		}
		return super.onKeyDown(keyCode, event);
	}

	private class MyWebChromeClient extends WebChromeClient {
		private ProgressBar mVideoProgressView;

		@Override
		public void onShowCustomView(View view,
				WebChromeClient.CustomViewCallback callback) {
			Log.i("", "here in on ShowCustomView");
			MdotMWebView.this.setVisibility(View.GONE);

			// if a view already exists then immediately terminate the new one
			if (mCustomView != null) {
				callback.onCustomViewHidden();
				return;
			}

			mCustomViewContainer.addView(view);
			mCustomView = view;
			mCustomViewCallback = callback;
			mCustomViewContainer.setVisibility(View.VISIBLE);

		}

		@Override
		public void onHideCustomView() {
			if (mCustomView == null)
				return;

			// Hide the custom view.
			mCustomView.setVisibility(View.GONE);

			// Remove the custom view from its container.
			mCustomViewContainer.removeView(mCustomView);
			mCustomView = null;
			mCustomViewContainer.setVisibility(View.GONE);
			mCustomViewCallback.onCustomViewHidden();

			MdotMWebView.this.setVisibility(View.VISIBLE);
			MdotMWebView.this.goBack();
		}

		@Override
		public View getVideoLoadingProgressView() {
			Log.i("", "here in on getVideoLoadingPregressView");

			if (mVideoProgressView == null) {
				mVideoProgressView = new ProgressBar(getContext());
				mVideoProgressView.setIndeterminate(false);
				android.widget.RelativeLayout.LayoutParams activityParams = new android.widget.RelativeLayout.LayoutParams(
						40, 40);
				mVideoProgressView.setLayoutParams(activityParams);
				mVideoProgressView.setVisibility(INVISIBLE);
				mVideoProgressView.setMinimumHeight(10);
				mVideoProgressView.setMinimumWidth(10);
				mVideoProgressView.setMax(100);
			}
			return mVideoProgressView;
		}

		@Override
		public void onReceivedTitle(WebView view, String title) {
			((Activity) mContext).setTitle(title);
		}

		@Override
		public void onProgressChanged(WebView view, int newProgress) {
			((Activity) mContext).getWindow().setFeatureInt(
					Window.FEATURE_PROGRESS, newProgress * 100);
		}

		@Override
		public void onGeolocationPermissionsShowPrompt(String origin,
				GeolocationPermissions.Callback callback) {
			callback.invoke(origin, true, false);
		}
	}

	static final FrameLayout.LayoutParams COVER_SCREEN_PARAMS = new FrameLayout.LayoutParams(
			ViewGroup.LayoutParams.MATCH_PARENT,
			ViewGroup.LayoutParams.MATCH_PARENT);
}
